% solves MSZ (2019):
% - one asset economy without renegotiation
%
% Note:
% - uses spline extrapolation (Matlab's built-in function fnxtr) 
% to evaluate the unkown functions outside and between the grid points.

%% 0. housekeeping
clear all
clc

fprintf('Replicating Figure B1... \n\n');

%% 1. setting parameters

% preferences
gamma = 2;                      % risk aversion parameter

% load parameters, functions, and solution of one asset economy with
% renegotiation
load(strcat('../One asset with renegotiation/solution_gamma',int2str(gamma)));

% save variables under different names
bgrid_R = bgrid; i0_R = i0; bh_R = bh; bl_R = bl; xh_R = xh; xl_R = xl; Psi_R = Psi;
ibhatl_Phi_R = ibhatl_Phi; ibhath_Phi_R = ibhath_Phi;
 
% set up the debt grid
N = size(bgrid,1);
bmax = bgrid_R(end);
bmin = bgrid_R(1);

% build up equally spaced grid around 0
i0            = ceil(N/10);           % number of asset grid points
Nb            = N-i0;                 % number of debt grid points
bgrid         = zeros(N,1);
bgrid(1:i0,:) = linspace(bmin,0,i0)';
%i0 will denote the index of the zero debt value
bgrid(i0:end,:) = linspace(0,bmax,Nb+1)';

S = size(phil,1);

% specify interpolation method
method = 'spline';

% load solution
load(strcat('solution_gamma',int2str(floor(gamma))),'bh','bl','xh','xl','bhath_Phi','ibhath_Phi','bhatl_Phi','ibhatl_Phi','Psi');
% assign value functions
Wh        = xh(:,2);
Wl        = xl(:,2);

% future debt is not allowed to exceed the general debt limit. allowing it
% would lead to the same allocation
bph = bh;
bpl = bh;
% use the same future grid
bp  = bh;

% define non-flat range for the interpolation
rangeh = 1:ibhath_Phi(end);
rangel = 1:ibhatl_Phi(end);

%% 2. solve for the stationary equilibrium (value function iteration)

% prepare outer loop
critout    = 10^(-12);
distout    = 1;
iterout    = 1;
maxiterout = 20;

while (iterout<=maxiterout)&&(distout>critout)

    %%% compute bond prices
    % indicates whether bp is bigger or equal than any of the bhat_Phi
    % - sum of I(NxS) along dimension 2 returns the index of the highest cost
    % realization which still induces repayment
    Ih  = (repmat(bp,1,S)>=repmat(bhath_Phi',N,1));
    Il  = (repmat(bp,1,S)>=repmat(bhatl_Phi',N,1));

    % default probability
    Fh  = Ih*pi;
    Fl  = Il*pi;

    % discounted recovery value
    QBhhat = 1/R*(bp.*(1-Fh)+Ih*(pi.*bgrid(i0)));   % default yields zero payoff!!
    QBlhat = 1/R*(bp.*(1-Fl)+Il*(pi.*bgrid(i0)));
    
    % prepare inner loop
    crit    = 10^(-12);                 % convergence criterion
    dist    = 1;                        % norm of the step
    iter    = 1;                        % iteration counter
    maxiter = 100;                      % maximum iterations
    
    % display innter loop, header
    fprintf('Solve for the stationary equilibrium: \n');
    fprintf('inner loop: \n');
    fprintf('iter \t norm \n');
    
    while (iter<=maxiter)&&(dist>crit)

    % conditional continuation values
    % note the difference between the threshold debt level and the renegotiated
    % debt level!
    % - matrix multiplication of I with with pi*fun(phi) yields probability weighted
    % cumulative sum
    EVh = (1-Fh).*Wh+Ih*(pi.*(Wh(i0)-phih));
    EVl = (1-Fl).*Wl+Il*(pi.*(Wl(i0)-phil));    % default payoff!!

    % update recovery probability
    Psi = min(1,max(0,X1inv(beta*(EVh-EVl))));

    % bond revenue
    QBh = ph*QBhhat+(1-ph).*QBlhat;
    QBl = Psi.*QBhhat+(1-Psi).*QBlhat;

    %%% set up matrix variables
    
    % normal times
    [bhmat,bphmat] = ndgrid(bh,bp);
    QBhmat         = repmat(QBh',N,1);
    chmat          = QBhmat+wh-bhmat;
    uhmat          = (chmat<=0)*(-10^(6))+(chmat>0).*u(chmat);    % takes care of negative consumption levels
    EVhhmat        = repmat(ph*EVh',N,1);                         % continuation in normal times
    EVhlmat        = repmat((1-ph)*EVl',N,1);                     % continuation in recession
    
    % search for the max along the row (or along bp)
    % limit search to the non-flat range
    [Whnext,ih] = max(uhmat(rangeh,rangeh)+beta*(EVhhmat(rangeh,rangeh)+EVhlmat(rangeh,rangeh)),[],2);
    
    % fill up the flat-part with end values
    Whnext(rangeh(end)+1:N) = Whnext(rangeh(end));
    ih(rangeh(end)+1:N)     = ih(rangeh(end));
    
    % update continuation value
    EVhnext = (1-Fh).*Whnext+Ih*(pi.*(Whnext(i0)-phih));
    
    [blmat,bplmat] = ndgrid(bl,bp);
    Psimat         = repmat(Psi',N,1);                              % recovery probability
    QBlmat         = repmat(QBl',N,1);
    clmat          = QBlmat+wl-blmat;
    ulmat          = (clmat<=0)*(-10^(6))+(clmat>0).*u(clmat)...    % takes care of negative consumption levels
                     -X(Psimat);
    EVlhmat        = repmat((Psi.*EVhnext)',N,1);                  % continuation in normal times
    EVllmat        = repmat(((1-Psi).*EVl)',N,1);                  % continuation in recession

    % search for the max along the row (or along bp)
    % limit search to the non-flat range
    [Wlnext,il] = max(ulmat(rangel,rangeh)+beta*(EVlhmat(rangel,rangeh)+EVllmat(rangel,rangeh)),[],2);
    
    % fill up the flat-part with end values
    Wlnext(rangel(end)+1:N)  = Wlnext(rangel(end));
    il(rangel(end)+1:N)      = il(rangel(end));
    
    % update continuation value
    EVlnext = (1-Fl).*Wlnext+Il*(pi.*(Wlnext(i0)-phil));    % default cost!!
    
    % Howard's improvement (policy function iterations)
    % choose number of steps
    % note: 
    % - chose step=0 to skip the policy function iterations
    %steps    = iter*0;              % number of steps
    steps    = 5;
    
    % define constant matrices
    QBhi = QBh(ih);
    bphi = bgrid(ih);
    
    QBli = QBl(il);
    bpli = bgrid(il);
    Psii = Psi(il);
    Xi   = X(Psii);

    for t=1:steps
        % update the value functions
        Whnext = u(QBhi+wh-bh)+beta*(ph*EVhnext(ih)+(1-ph).*EVlnext(ih));
        Wlnext = u(QBli+wl-bl)-Xi+beta*(Psii.*EVhnext(il)+(1-Psii).*EVlnext(il));
        
        % update the continuation values
        EVhnext = (1-Fh).*Whnext+Ih*(pi.*(Whnext(i0)-phih));
        EVlnext = (1-Fl).*Wlnext+Il*(pi.*(Wlnext(i0)-phil));  % default cost!!
    end

    % compute distance
    dist = norm([Wh-Whnext,Wl-Wlnext]);

    % update the value function
    Wh  = Whnext;
    Wl  = Wlnext;

    % display inner loop, progress
    fprintf('%4i \t %2.1e \n',iter,dist);

    % increase the iteration counter
    iter = iter+1;

    end
    
    %%% normal times: update bhat_Phi according to the threshold conditions
    
    Whhat_Phi = Wh(i0)-phih;
    Whhat     = Wh(i0)-phih;
    
    % interpolate WChhat_Phi on (bh,WCh) to retrieve bhath_Phinext
    bhath_Phinext = fnval(Whhat_Phi,fnxtr(csaps(Wh(rangeh),bh(rangeh)),2));
                
    % returns the index of the last grid point smaller than bhat_Phinext
    % max make sure the index is in a valid range
    ibhath_Phinext = max(1,sum(repmat(bhath_Phinext,1,N)>=repmat(bgrid',S,1),2));
    bhath_Phinext  = bgrid(ibhath_Phinext);
    
    %%% recession: update bhat_Phi and bhat according to the threshold conditions
    
    WClhat_Phi = Wl(i0)-phil;
    WClhat     = Wl(i0)-phil;

    % interpolate WChhat_Phi on (bh,WCh) to retrieve bhath_Phinext
    bhatl_Phinext = fnval(WClhat_Phi,fnxtr(csaps(Wl(rangel),bl(rangel)),2));
                                
    % returns the index of the last grid point smaller than bhat_Phinext
    % max make sure the index is in a valid range
    ibhatl_Phinext = max(1,sum(repmat(bhatl_Phinext,1,N)>=repmat(bgrid',S,1),2));
    bhatl_Phinext  = bgrid(ibhatl_Phinext);
    
    % truncate debt grids according to the new renegotiation
    % threshold
    bhnext = min(bgrid,bhath_Phinext(end));
    blnext = min(bgrid,bhatl_Phinext(end));
    
    % compute the convergence criterion
    distout = norm([bhath_Phinext-bhath_Phi;...
                    bhatl_Phinext-bhatl_Phi;...
                    Whnext-Wh;...
                    Wlnext-Wl]);
                
    % update all normal times objects
    Wh        = Whnext;
    bhath_Phi  = bhath_Phinext;
    ibhath_Phi = ibhath_Phinext;
    bh         = bhnext;
    bph        = bh;
    
    % update non-flat range for the interpolation
    rangeh = 1:ibhath_Phi(end);
                
    % update all recession objects
    Wl         = Wlnext; 
    bhatl_Phi  = bhatl_Phinext;
    ibhatl_Phi = ibhatl_Phinext;
    bl         = blnext;
    bpl        = bh;
    
    % update non-flat range for the interpolation
    rangel = 1:ibhatl_Phi(end);

    % display outer loop, progress
    fprintf('Outer loop: \n');
    fprintf('%4i \t %2.1e \n',iterout,distout);

    iterout = iterout+1;
    
end

fprintf('Done with stationary equilibrium. \n');

%% 4. compute expected values given initial debt and save solution

% compute recovery value (non-disounted) for given debt after the realization
% of w and before the realization of the default cost.

Pih_NR = R*QBhhat;
Pil_NR = R*QBlhat;

% expected utility for given debt after the realization of w and
% before the realization of the default cost.

EVh_NR = EVh;
EVl_NR = EVl;

% save solution
chi = QBh(ih)-bh+wh;
cli = QBl(il)-bl+wl;
xh  = [bph(ih),Wh,chi,QBh];
xl  = [bpl(il),Wl,cli,QBl,Psii];

% uncomment if you want to overwrite the saved solution
% save(strcat('solution_gamma',int2str(gamma)),'bgrid','i0','bh','bl','xh','xl','bhath_Phi','ibhath_Phi','bhatl_Phi','ibhatl_Phi','Psi','Psii',...
%                                              'beta','ph','phil','phih','pi','R','wh','wl','u','uinv','u1','u1inv','X','X1','X11','X1inv',...
%                                              'Pih_NR','Pil_NR','EVh_NR','EVl_NR');

%% 5. plot policy functions
%%% debt accumulation and effort: One asset economy with Renegotiation vs. No Renegotiation

% load policy functions of one asset economy with renegotiation
load('../One asset with renegotiation/policy_functions','bl_R','bpli_R','cl_R','Psii_R','bhatl_R','bhath_R','bhatl_Phi_R','bhath_Phi_R','Fl_R','Fh_R');

% identify discontinuities
dis       = diff(bpli_R);
% identifies 3 significant discontinuities > .015
dis_index = find(dis>.015);

scrsz = get(0,'ScreenSize');
figure('Position',[scrsz(3)*1/8+40 scrsz(4)*1/8-50 scrsz(3)*1/2 ...
    scrsz(4)*3/5]);

% debt accumulation
subplot(2,2,1)

% smoothen plot
bbpli = fnval(bl,fnxtr(csaps(bl,bpli,.99),2));

hh = plot(bl,bbpli,bl_R,[bpli_R,bl_R]);
set(hh(1),'Color','b','LineWidth',1.5);
set(hh(2),'Color',.5*[0,1,0],'LineWidth',1.5,'LineStyle','--');
set(hh(3),'Color','k','LineWidth',1','LineStyle','-.');
hh(4) = line([max(bl_R),max(bl_R)],[0,max(bpli_R)],...
         'LineWidth',1.25,'color','k','linestyle',':');
hh(5) = line([max(bl),max(bl)],[0,max(bpli_R)],...
         'LineWidth',1.25,'color','k','linestyle',':');
%%% start: discontinuities
hold on;
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [bpli_R(dis_index(1)),bpli_R(dis_index(1)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [bpli_R(dis_index(1)),bpli_R(dis_index(1)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [bpli_R(dis_index(2)),bpli_R(dis_index(2)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [bpli_R(dis_index(2)),bpli_R(dis_index(2)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [bpli_R(dis_index(3)),bpli_R(dis_index(3)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [bpli_R(dis_index(3)),bpli_R(dis_index(3)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
hold off;
%%% end: discontinuities
title('Debt accumulation');
xlim([0 max(bl_R)+.05]);
ylim([0-.01,max(bpli_R)+.01])
xlabel('Current debt, b'); ylabel('$B(b)$','interpreter','latex');
legend([hh(1) hh(2) hh(3)],{'No Renegotiation','Noncontingent',...
                    '$45^{\circ}$-line'},...
                    'interpreter','latex','Location','NorthWest');
                
set(gca,'XTick',[0,0.25,0.5,max(bl),1,max(bl_R)]);
xticks        = get(gca,'XTickLabel');
xticks{end-2} = 'b^{max}_{NR}';
xticks{end}   = 'b^{max}_{R}';
set(gca,'XTickLabel',xticks);

% reform effort
subplot(2,2,2)

% smoothen plot
PPsii = fnval(bl,fnxtr(csaps(bl,Psii,.99),2));

hh = plot(bl,PPsii,bl_R,Psii_R);
set(hh(1),'Color','b','LineWidth',1.5);
set(hh(2),'Color',.5*[0,1,0],'LineWidth',1.5,'LineStyle','--');
line([max(bl_R) max(bl_R)],[Psii_R(end)-.002 max(Psii)+.05],'Linewidth',1.25,'Linestyle',':','Color','k');
line([max(bl),max(bl)],[0,max(bpli_R)],...
         'LineWidth',1.25,'color','k','linestyle',':');
%%% start: discontinuities
hold on;
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [Psii_R(dis_index(1)),Psii_R(dis_index(1)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [Psii_R(dis_index(1)),Psii_R(dis_index(1)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [Psii_R(dis_index(2)),Psii_R(dis_index(2)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [Psii_R(dis_index(2)),Psii_R(dis_index(2)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [Psii_R(dis_index(3)),Psii_R(dis_index(3)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [Psii_R(dis_index(3)),Psii_R(dis_index(3)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
hold off;
%%% end: discontinuities
title('Reform effort')
ylabel('$\Psi(B(b))$','interpreter','latex');
xlabel('Current debt, b')
xlim([0,max(bl_R)+.05]);
ylim([Psii_R(end)-.002,max(Psii)+.002]);

set(gca,'XTick',[0,0.25,0.5,max(bl),1,max(bl_R)]);
xticks        = get(gca,'XTickLabel');
xticks{end-2} = 'b^{max}_{NR}';
xticks{end}   = 'b^{max}_{R}';
set(gca,'XTickLabel',xticks);

% current consumption
subplot(2,2,3)

% smoothen plot
ccli = fnval(bl,fnxtr(csaps(bl,cli,.99),2));

hh = plot(bl,ccli,bl_R,cl_R);
set(hh(1),'Color','b','LineWidth',1.5);
set(hh(2),'Color',[0 .5 0],'LineWidth',1.5,'LineStyle','--');
line([max(bl_R),max(bl_R)],[0,min(cl_R(i0_R))+.05],...
        'LineWidth',1.25,'color','k','linestyle',':');
line([max(bl),max(bl)],[0,cl_R(i0_R)+.05],...
        'LineWidth',1.25,'color','k','linestyle',':');
%%% start: discontinuities
% note: consumption needs a lower threshold
% identifies 6 significant discontinuities > .05
dis_index = find(dis>.005);
hold on;
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [cl_R(dis_index(1)),cl_R(dis_index(1)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [cl_R(dis_index(1)),cl_R(dis_index(1)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [cl_R(dis_index(2)),cl_R(dis_index(2)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [cl_R(dis_index(2)),cl_R(dis_index(2)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [cl_R(dis_index(3)),cl_R(dis_index(3)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [cl_R(dis_index(3)),cl_R(dis_index(3)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(4)),bl_R(dis_index(4))],...
      [cl_R(dis_index(4)),cl_R(dis_index(4)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(4)),bl_R(dis_index(4))],...
      [cl_R(dis_index(4)),cl_R(dis_index(4)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(5)),bl_R(dis_index(5))],...
      [cl_R(dis_index(5)),cl_R(dis_index(5)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(5)),bl_R(dis_index(5))],...
      [cl_R(dis_index(5)),cl_R(dis_index(5)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(6)),bl_R(dis_index(6))],...
      [cl_R(dis_index(6)),cl_R(dis_index(6)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(6)),bl_R(dis_index(6))],...
      [cl_R(dis_index(6)),cl_R(dis_index(6)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
hold off;
%%% end: discontinuities
title('Current consumption');
xlabel('Current debt, b'); ylabel('$C(b)$','interpreter','latex');
xlim([0 max(bl_R)+.05]);
ylim([min(cli)-.05,cl_R(i0_R)+.05]);
%legend('benchmark','reform','Location','Best');

set(gca,'XTick',[0,0.25,0.5,max(bl),1,max(bl_R)]);
xticks        = get(gca,'XTickLabel');
xticks{end-2} = 'b^{max}_{NR}';
xticks{end}   = 'b^{max}_{R}';
set(gca,'XTickLabel',xticks);

% probability of repayment
subplot(2,2,4)
hh = plot(bl,1-Fl,bl_R,1-Fl_R);
set(hh(1),'Color','b','LineWidth',1.5);
set(hh(2),'Color',[0 .5 0],'LineWidth',1.5,'LineStyle','--');
line([max(bl_R),max(bl_R)],[0,1],...
        'LineWidth',1.25,'color','k','linestyle',':');
line([max(bl),max(bl)],[0,1],...
        'LineWidth',1.25,'color','k','linestyle',':');
title('Probability of repayment');
xlabel('Current debt, b'); ylabel('$1-F(\Phi(b))$','interpreter','latex');
xlim([0 max(bl_R)+.05]);
ylim([0-.01,1+.01]);

set(gca,'XTick',[0,0.25,0.5,max(bl),1,max(bl_R)]);
xticks        = get(gca,'XTickLabel');
xticks{end-2} = 'b^{max}_{NR}';
xticks{end}   = 'b^{max}_{R}';
set(gca,'XTickLabel',xticks);
