% solves MSZ (2019):
% - one-asset economy with renegotiation
%
% Note:
% - uses spline extrapolation (Matlab's built-in function fnxtr) 
% to evaluate the unkown functions outside and between the grid points.

%% 0. housekeeping
clear all
clc

%% 1. parameters and functional forms

% flag for extended debt grid
% note: SB=1 is only for comparison with second-best allocations
SB = 0;

%%% parameters

% technology
R     = 1.02;                           % gross interest rate
beta  = 1/R;                            % subjective discount factor

% preferences
gamma = 2;                              % relative risk aversion

% effort cost function
xi     = 14.3713;
psi    = 10;

% default cost shocks
% inverse exponential distribution with truncation point, phimax
phimax = 2.2745;
eta    = 1.6247;
    
% income process
wh    = 1;                               % income in good state (normal times)
drop  = 0.25;                            % drop (in %) in income in recession
wl    = (1-drop)*wh;                     % income in the bad state (recession)

% absorbing recovery                            
ph    = 1;

%%% functions

% utility function (CRRA no constant)
if gamma ~= 1
    u    = @(c) (c.^(1-gamma))/(1-gamma);
    uinv = @(v) (v*(1-gamma)).^(1/(1-gamma));
else
    u    = @(c) log(c);                 % log-utility
    uinv = @(v) exp(v);                 % inverse
end

% marginal utility function
u1    = @(c) c.^(-gamma);
u1inv = @(v) v.^(-1/gamma);

% effort cost function
X      = @(p) xi*p.^(1+1/psi)/(1+1/psi);
X1     = @(p) xi*p.^(1/psi);
X11    = @(p) xi/psi*p.^(1/psi-1);
X1inv  = @(x) min(1,(max(0,x)/xi).^(psi));            % inverse of X1(p)

%%% discretization of the outstanding debt

% set up grid for debt
N = 5000;                               % number of grid points

if SB==0
    % choose grid consistent with the calibration target for the debt
    % limit.
    bmax   = 1.85*wh;
    bmin   = -.1;
elseif SB==1
    % extended grid for comparison with second-best economies
    bmax   = 7.5;
    bmin   = -.1;
end

% build up equally spaced grid around 0
i0              = ceil(N/10);           % 10% of the grid points are *asset* grid points
Nb              = N-i0;                 % the rest are *debt* grid points
bgrid           = zeros(N,1);
bgrid(1:i0,:)   = linspace(bmin,0,i0)';
% i0 will denote the index of the zero debt/asset value
bgrid(i0:end,:) = linspace(0,bmax,Nb+1)';

%%% discretization of the default cost distribution
S      = 600;                           % number of grid points
phimin = 0;                             % minimum default cost realization

% use non-equally spaced grid such that cumulative sums reflect the
% distributional assumption.

% (phibar-phi) is exponential with rate parameter eta and truncation
% point phimax

% set up distribution functions
mypdf    = @(x,xbar,eta) eta*exp(-eta*(xbar-x))/(1-exp(-eta*xbar));
mycdf    = @(x,xbar,eta) (exp(-eta*(xbar-x))-exp(-eta*xbar))/(1-exp(-eta*xbar));
mycdfinv = @(P,xbar,eta) xbar+log((1-exp(-eta*xbar))*P+exp(-eta*xbar))/eta;

% set up inverse cumulative distribution function
phil = mycdfinv(linspace(mycdf(phimin,phimax,eta),mycdf(phimax,phimax,eta),S)',phimax,eta);

% assign uniform state probabilities
pi  = 1/S*ones(S,1);

% commitment in normal times
phih = 10^(6)*ones(size(phil));             % choose a very high cost realization

% specify interpolation method
method = 'spline';

% load solution
if SB==0
    load(strcat('solution_gamma',int2str(floor(gamma))),'bh','bl','xh','xl','bhath_Phi','ibhath_Phi','bhath','ibhath','bhatl_Phi','ibhatl_Phi','bhatl','ibhatl','Psi');
elseif SB==1
    load(strcat('solution_gamma',int2str(floor(gamma)),'_SB'),'bh','bl','xh','xl','bhath_Phi','ibhath_Phi','bhath','ibhath','bhatl_Phi','ibhatl_Phi','bhatl','ibhatl','Psi'); 
end
Wh        = xh(:,2);
Wl        = xl(:,2);

% future debt does not exceed the general debt limit
bph = bh;
bpl = bh;
% use the same grid for debt issuance
bp  = bh;

% define non-flat range on the grid for the interpolation
rangeh = 1:ibhath_Phi(end);
rangel = 1:ibhatl_Phi(end);

%% 2. solve for the stationary equilibrium (value function iteration)

% prepare outer loop
critout    = 10^(-12);
distout    = 1;
iterout    = 1;
maxiterout = 50;

while (iterout<=maxiterout)&&(distout>critout)

    %%% compute bond prices
    % indicates whether bp is bigger or equal than any of the bhat_Phi
    % - sum of I(NxS) along dimension 2 returns the index of the highest cost
    % realization which still induces renegotiation
    Ih  = (repmat(bp,1,S)>=repmat(bhath_Phi',N,1));
    Il  = (repmat(bp,1,S)>=repmat(bhatl_Phi',N,1));

    % default probability
    Fh  = Ih*pi;
    Fl  = Il*pi;

    % discounted recovery value
    QBhhat = 1/R*(bp.*(1-Fh)+Ih*(pi.*bhath));
    QBlhat = 1/R*(bp.*(1-Fl)+Il*(pi.*bhatl));
    
    % prepare inner loop
    crit    = 10^(-12);                 % convergence criterion
    dist    = 1;                        % norm of the step
    iter    = 1;                        % iteration counter
    maxiter = 20;                       % maximum iterations
    
    % display innter loop, header
    fprintf('Solve for the stationary equilibrium: \n');
    fprintf('inner loop:\n');
    fprintf('iter \t norm \n');
    
    while (iter<=maxiter)&&(dist>crit)

    % conditional continuation values
    % note the difference between the threshold debt level and the renegotiated
    % debt level!
    % - matrix multiplication of I with with pi*fun(phi) yields probability weighted
    % cumulative sum
    EVh = (1-Fh).*Wh+Ih*(pi.*Wh(ibhath));
    EVl = (1-Fl).*Wl+Il*(pi.*Wl(ibhatl));

    % update recovery probability
    Psi = min(1,max(0,X1inv(beta*(EVh-EVl))));

    % bond revenue
    QBh = ph*QBhhat+(1-ph).*QBlhat;
    QBl = Psi.*QBhhat+(1-Psi).*QBlhat;

    %%% set up matrix variables
    
    % normal times
    [bhmat,bphmat] = ndgrid(bh,bp);
    QBhmat         = repmat(QBh',N,1);
    chmat          = QBhmat+wh-bhmat;
    uhmat          = (chmat<=0)*(-10^(6))+(chmat>0).*u(chmat);    % takes care of negative consumption levels
    EVhhmat        = repmat(ph*EVh',N,1);                         % continuation in normal times
    EVhlmat        = repmat((1-ph)*EVl',N,1);                     % continuation in recession
    
    % search for the max along the row (or along bp)
    % limit search to the non-flat range
    [Whnext,ih] = max(uhmat(rangeh,rangeh)+beta*(EVhhmat(rangeh,rangeh)+EVhlmat(rangeh,rangeh)),[],2);
    
    % fill up the flat-part with end values
    Whnext(rangeh(end)+1:N) = Whnext(rangeh(end));
    ih(rangeh(end)+1:N)     = ih(rangeh(end));
    
    % update continuation value
    EVhnext = (1-Fh).*Whnext+Ih*(pi.*Whnext(ibhath));
    
    [blmat,bplmat] = ndgrid(bl,bp);
    Psimat         = repmat(Psi',N,1);                              % recovery probability
    QBlmat         = repmat(QBl',N,1);
    clmat          = QBlmat+wl-blmat;
    ulmat          = (clmat<=0)*(-10^(6))+(clmat>0).*u(clmat)...    % takes care of negative consumption levels
                     -X(Psimat);
    EVlhmat        = repmat((Psi.*EVhnext)',N,1);                  % continuation in normal times
    EVllmat        = repmat(((1-Psi).*EVl)',N,1);                  % continuation in recession

    % search for the max along the row (or along bp)
    % limit search to the non-flat range
    [Wlnext,il] = max(ulmat(rangel,rangeh)+beta*(EVlhmat(rangel,rangeh)+EVllmat(rangel,rangeh)),[],2);
    
    % fill up the flat-part with end values
    Wlnext(rangel(end)+1:N)  = Wlnext(rangel(end));
    il(rangel(end)+1:N)      = il(rangel(end));
    
    % update continuation value
    EVlnext = (1-Fl).*Wlnext+Il*(pi.*Wlnext(ibhatl));
    
    % Howard's improvement (policy function iterations)
    % choose number of steps
    % note: 
    % - chose step=0 to skip the policy function iterations
    %steps    = iter*0;              % number of steps
    steps = 0;

    % define constant matrices
    QBhi = QBh(ih);
    bphi = bgrid(ih);
    
    QBli = QBl(il);
    bpli = bgrid(il);
    Psii = Psi(il);
    Xi   = X(Psii);

    for t=1:steps
        % update the value functions
        Whnext = u(QBhi+wh-bh)+beta*(ph*EVhnext(ih)+(1-ph).*EVlnext(ih));
        Wlnext = u(QBli+wl-bl)-Xi+beta*(Psii.*EVhnext(il)+(1-Psii).*EVlnext(il));
        
        % update the continuation values
        EVhnext = (1-Fh).*Whnext+Ih*(pi.*Whnext(ibhath));
        EVlnext = (1-Fl).*Wlnext+Il*(pi.*Wlnext(ibhatl));
    end

    % compute distance
    dist = norm([Wh-Whnext,Wl-Wlnext]);

    % update the value function
    Wh  = Whnext;
    Wl  = Wlnext;

    % display inner loop, progress
    fprintf('%4i \t %2.1e \n',iter,dist);

    % increase the iteration counter
    iter = iter+1;

    end
    
    %%% normal times: update bhat_Phi and bhat according to the threshold conditions
    
    Whhat_Phi = Wh(i0)-phih;
    Whhat     = Wh(i0)-phih;
    
    % interpolate WChhat_Phi on (bh,WCh) to retrieve bhath_Phinext
    bhath_Phinext = fnval(Whhat_Phi,fnxtr(csaps(Wh(rangeh),bh(rangeh)),2));
    
    % interpolate WChhat on (bh,WCh) to retrieve bhathnext
    bhathnext = fnval(Whhat,fnxtr(csaps(Wh(rangeh),bh(rangeh)),2));
                
    % returns the index of the last grid point smaller than bhat_Phinext
    % max make sure the index is in a valid range
    ibhath_Phinext = max(1,sum(repmat(bhath_Phinext,1,N)>=repmat(bgrid',S,1),2));
    bhath_Phinext  = bgrid(ibhath_Phinext);
    ibhathnext     = max(1,sum(repmat(bhathnext,1,N)>=repmat(bgrid',S,1),2));
    bhathnext      = bgrid(ibhathnext);
    
    %%% recession: update bhat_Phi and bhat according to the threshold conditions
    
    WClhat_Phi = Wl(i0)-phil;
    WClhat     = Wl(i0)-phil;

    % interpolate WChhat_Phi on (bh,WCh) to retrieve bhath_Phinext
    bhatl_Phinext = fnval(WClhat_Phi,fnxtr(csaps(Wl(rangel),bl(rangel)),2));
    
    % interpolate WChhat on (bh,WCh) to retrieve bhathnext
    bhatlnext = fnval(WClhat,fnxtr(csaps(Wl(rangel),bl(rangel)),2));
                                
    % returns the index of the last grid point smaller than bhat_Phinext
    % max make sure the index is in a valid range
    ibhatl_Phinext = max(1,sum(repmat(bhatl_Phinext,1,N)>=repmat(bgrid',S,1),2));
    bhatl_Phinext  = bgrid(ibhatl_Phinext);
    ibhatlnext     = max(1,sum(repmat(bhatlnext,1,N)>=repmat(bgrid',S,1),2));
    bhatlnext      = bgrid(ibhatlnext);
    
    % truncate debt grids according to the new renegotiation
    % threshold
    bhnext = min(bgrid,bhath_Phinext(end));
    blnext = min(bgrid,bhatl_Phinext(end));
    
    % compute the convergence criterion
    distout = norm([bhath_Phinext-bhath_Phi;bhathnext-bhath;...
                    bhatl_Phinext-bhatl_Phi;bhatlnext-bhatl;...
                    Whnext-Wh;...
                    Wlnext-Wl]);
                
    % update all normal times objects
    Wh        = Whnext;
    bhath_Phi  = bhath_Phinext;
    ibhath_Phi = ibhath_Phinext;
    bhath      = bhathnext;
    ibhath     = ibhathnext;
    bh         = bhnext;
    bph        = bh;
    
    % update non-flat range for the interpolation
    rangeh = 1:ibhath_Phi(end);
                
    % update all recession objects
    Wl         = Wlnext; 
    bhatl_Phi  = bhatl_Phinext;
    ibhatl_Phi = ibhatl_Phinext;
    bhatl      = bhatlnext;
    ibhatl     = ibhatlnext;
    bl         = blnext;
    bpl        = bh;
    
    % update non-flat range for the interpolation
    rangel = 1:ibhatl_Phi(end);

    % display outer loop, progress
    fprintf('Outer loop: \n');
    fprintf('%4i \t %2.1e \n',iterout,distout);

    iterout = iterout+1;
    
end

fprintf('Done. \n');

%% 3. compute expected values given initial debt and save solution

% compute recovery value (non-disounted) for given debt after the realization
% of w and before the realization of the default cost.

Pih_R = R*QBhhat;
Pil_R = R*QBlhat;

% expected utility for given debt after the realization of w and
% before the realization of the default cost.

EVh_R = EVh;
EVl_R = EVl;

% save solution
chi = QBh(ih)-bh+wh;
cli = QBl(il)-bl+wl;
xh  = [bph(ih),Wh,chi,QBh];
xl  = [bpl(il),Wl,cli,QBl,Psii];

% % uncomment if you want to overwrite the saved solution
% if SB==0
%     save(strcat('solution_gamma',int2str(gamma)),'bgrid','i0','bh','bl','xh','xl','bhath_Phi','ibhath_Phi','bhath','ibhath','bhatl_Phi','ibhatl_Phi','bhatl','ibhatl','Psi','Psii',...
%                                                  'beta','ph','phil','phih','pi','R','wh','wl','u','uinv','u1','u1inv','X','X1','X11','X1inv',...
%                                                  'Pih_R','Pil_R','EVh_R','EVl_R');
% elseif SB==1
%     save(strcat('solution_gamma',int2str(gamma),'_SB'),'bgrid','i0','bh','bl','xh','xl','bhath_Phi','ibhath_Phi','bhath','ibhath','bhatl_Phi','ibhatl_Phi','bhatl','ibhatl','Psi','Psii',...
%                                                  'beta','ph','phil','phih','pi','R','wh','wl','u','uinv','u1','u1inv','X','X1','X11','X1inv',...
%                                                  'Pih_R','Pil_R','EVh_R','EVl_R');
% 
% end

%% 4. save policy functions

% save policy function for comparison with two-asset economy with moral
% hazard and the one-asset economy with renegotiation
bl_R        = bl;
bpli_R      = bpl(il);
cl_R        = cli;
Psii_R      = Psii;
bhatl_R     = bhatl;
bhath_R     = bhath;
bhatl_Phi_R = bhatl_Phi;
bhath_Phi_R = bhath_Phi;
Fl_R        = Fl;
Fh_R        = Fh;

% uncomment if you want to overwrite the saved policy functions
% if SB==0
%     save('policy_functions','bl_R','bpli_R','cl_R','Psii_R','bhatl_R','bhath_R','bhatl_Phi_R','bhath_Phi_R','Fl_R','Fh_R');
% end
