%%% solves MSZ (2019):
% - decentralization of the planner allocation w/o moral hazard
% - two asset economy with renegotiation
%
% Note:
% - uses spline extrapolation (Matlab's built-in function fnxtr) 
% to evaluate the unkown functions outside and between the grid points.

%% 0. housekeeping
clear all
clc

%% 1. setting parameters and choosing the grid

% preferences
gamma = 2;                      % risk aversion parameter

% load parameters, functions, and solution of the one-asset economy with the
% extended grid
load(strcat('../../One asset with renegotiation/solution_gamma',int2str(gamma),'_SB'));

% save variables of one asset economy under different names
bgrid_R = bgrid; bh_R = bh; bl_R = bl; xh_R = xh; xl_R = xl; Psi_R = Psi; Psii_R = Psii;
ibhatl_Phi_R = ibhatl_Phi; ibhath_Phi_R = ibhath_Phi;

% allows for a coarser debt grid as the curse of dimensionality starts
% kicking in with the choice of two debt levels
N = 1000;

% number of shock states;
S = size(phil,1);

% setup new (coarser) debt grid with the same span
bmax   = bh_R(end);
bmin   = bh_R(1);

% build up equally spaced grid around 0
i0              = ceil(N/10);           % number of asset grid points
Nb              = N-i0;                 % number of debt grid points
bgrid           = zeros(N,1);
bgrid(1:i0,:)   = linspace(bmin,0,i0)';
% i0 will denote the index of the zero debt value
bgrid(i0:end,:) = linspace(0,bmax,Nb+1)';

% specify interpolation method
method = 'spline';

% load solution
load(strcat('solution_gamma',int2str(gamma)),'bh','bl','xl','xh','Psi','bhath_Phi','ibhath_Phi','bhath','ibhath',...
                                             'bhatl_Phi','ibhatl_Phi','bhatl','ibhatl','Psi');
% assign value functions
Wh        = xh(:,3);
Wl        = xl(:,3);

% choose grid for the future debt
% starting in normal times
bph  = bh;
bphh = bh;
bphl = bl;

% starting in recession
bplh = bh;
bpll = bl;       % recession grid is smaller

% define non-flat ranges for interpolation
rangeh = 1:ibhath_Phi(end);
rangel = 1:ibhatl_Phi(end);

%% 2. solve for the equilibrium (value function iteration)

% prepare outer loop
critout    = 10^(-12);
distout    = 1;
iterout    = 1;
maxiterout = 100;

fprintf('Solve for the stationary equilibrium: \n');

while (iterout<=maxiterout)&&(distout>critout)

    % continuation in boom
    % indicates whether bp is bigger or equal than the renegotiation threshold, bhat_Phi
    Ih = @(bp) (repmat(bp,1,S)>=repmat(bhath_Phi',N,1));
    Il = @(bp) (repmat(bp,1,S)>=repmat(bhatl_Phi',N,1));

    % default probability
    Fh  = @(bp) Ih(bp)*pi;
    Fl  = @(bp) Il(bp)*pi;

    % interpolation ready value functions
    Whfun = @(bp) fnval(bp,fnxtr(csaps(bh(rangeh),Wh(rangeh)),2));
    Wlfun = @(bp) fnval(bp,fnxtr(csaps(bl(rangel),Wl(rangel)),2));
    
    % recovery values
    QBhhat = @(bp) 1/R*(bp.*(1-Fh(bp))+Ih(bp)*(pi.*bhath));
    QBlhat = @(bp) 1/R*(bp.*(1-Fl(bp))+Il(bp)*(pi.*bhatl));
    
    % matrix form allowing to search over all possible combinations of future
    % debt accumulation
    QBhhmat = repmat(QBhhat(bphh)',N,1);       % varies with bpxh along rows
    QBhlmat = repmat(QBlhat(bphl),1,N);        % varies with bpxl along columns

    QBlhmat = repmat(QBhhat(bplh)',N,1);       % varies with bpxh along rows
    QBllmat = repmat(QBlhat(bpll),1,N);        % varies with bpxl along columns

    % prepare inner loop
    crit    = 10^(-12);                 % convergence criterion
    dist    = 1;                        % norm of the step
    iter    = 1;                        % iteration counter
    maxiter = 100;                       % maximum iterations

    % display innter loop, header
    fprintf('inner loop: \n');
    fprintf('iter \t norm \n');

    while (dist>crit)&&(iter<=maxiter) 

    % note: uses the property that the debt accumulation function will be
    % (weakly) increasing in initial debt
    % speeds up the code substantially when working with large N

    % initialize variables
    
    % normal times
    Whnext  = zeros(N,1);
    ihh     = zeros(N,1);
    ihl     = zeros(N,1);
    QBhhi   = ones(N,1);
    QBhli   = ones(N,1);
    
    % recession
    Wlnext  = zeros(N,1);
    ilh     = zeros(N,1);
    ill     = zeros(N,1);
    QBlhi   = ones(N,1);
    QBlli   = ones(N,1);
    cli     = ones(N,1);
    EVlhi   = ones(N,1);
    EVlli   = ones(N,1);
    bplhi_exact   = ones(N,1);

    %%% normal times

    % continuation values

    % varies along the row (or along bpxh)
    EVhhmat = repmat(...
                ((1-Fh(bphh)).*Whfun(bphh)+Ih(bphh)*(pi.*Whfun(bhath)))'...
            ,N,1);

    % varies along the column (or along bpxl)
    EVhlmat = repmat(...
                (1-Fl(bphl)).*Wlfun(bphl)+Il(bphl)*(pi.*Wlfun(bhatl))...
            ,1,N);

    % start with the highest initial debt
    kh = ibhath_Phi(end);

    % consumption
    cmat = ph.*QBhhmat+(1-ph).*QBhlmat+wh-bh(kh);

    % momentary utility
    umat = (cmat<=0)*(-10^(6))+(cmat>0).*u(cmat); 

    % ! note: use knowledge from the FOC here
    % - constant debt, bh=bphh
    % - bphl irrelevant
    Whnext(kh) = umat(kh,kh)+beta*(ph*EVhhmat(kh,kh))+(1-ph)*EVhlmat(kh,kh);
    ihh(kh) = kh;
    ihl(kh) = kh;
    
    % recovery value at the optimal choice
    QBhhi(kh) = QBhhmat(ihl(kh),ihh(kh));
    QBhli(kh) = QBhlmat(ihl(kh),ihh(kh));

    % work through the remaining lower debt levels
    for kh =(ibhath_Phi(end)-1):-1:1

        % consumption and momentary utility
        cmat   = ph.*QBhhmat+(1-ph).*QBhlmat+wh-bh(kh);
        umat   = (cmat<=0)*(-10^(6))+(cmat>0).*u(cmat);      % takes care of negative consumption levels

        % use FOC knowledge here:
        Whnext(kh) = umat(kh,kh)+beta*(ph*EVhhmat(kh,kh))+(1-ph)*EVhlmat(kh,kh);
        ihh(kh) = kh;
        ihl(kh) = kh;   % irrelevant with absorbing recovery

        % bond revenue at the optimal choice
        QBhhi(kh) = QBhhmat(ihl(kh),ihh(kh));
        QBhli(kh) = QBhlmat(ihl(kh),ihh(kh));

    end

    % fill up the remaining values
    Whnext(ibhath_Phi(end)+1:end)  = Whnext(ibhath_Phi(end));
    ihl(ibhath_Phi(end)+1:end)     = ihl(ibhath_Phi(end));
    ihh(ibhath_Phi(end)+1:end)     = ihh(ibhath_Phi(end));
    QBhhi(ibhath_Phi(end)+1:end)   = QBhhi(ibhath_Phi(end));
    QBhli(ibhath_Phi(end)+1:end)   = QBhli(ibhath_Phi(end));

    % update the value function
    Whfun = @(bp) fnval(bp,fnxtr(csaps(bh(rangeh),Whnext(rangeh)),2));
    
    %%% recession
    
    % varies along the row (or along bplh)
    EVlhmat = repmat(...
                ((1-Fh(bplh)).*Whfun(bplh)+Ih(bplh)*(pi.*Whfun(bhath)))'...
            ,N,1);

    % varies along the column (or along bpll)
    EVllmat = repmat(...
                (1-Fl(bpll)).*Wlfun(bpll)+Il(bpll)*(pi.*Wlfun(bhatl))...
            ,1,N);

    % start with the highest initial debt
    kl = ibhatl_Phi(end);

    % consumption
    cmat   = Psi(kl).*QBlhmat(kl:rangel(end),kl:rangeh(end))+(1-Psi(kl)).*QBllmat(kl:rangel(end),kl:rangeh(end))+wl-bl(kl);  

    % momentary utility
    umat   = (cmat<=0)*(-10^(6))+(cmat>0).*(u(cmat)-X(Psi(kl)));      % takes care of negative consumption levels
    
    % note: use knowledge from the FOC here
    % - constant debt, bl=bpll
    Wlcond = umat(1,:)+beta*(Psi(kl).*EVlhmat(kl,kl:rangeh(end))+(1-Psi(kl)).*EVllmat(kl,kl:rangeh(end)));
    ill(kl) = 1;
    % optimal debt, bplh
    bplhi_exact(kl) = ((1-Psi(kl))*QBllmat(kl,1)+(wl-wh)-bl(kl))/(1/R-1-Psi(kl)/R);
    % get index (this is not precise because of the grid)
    ilh(kl)    = sum(bplhi_exact(kl)>=bgrid)-(kl-1);
    Wlnext(kl) = Wlcond(ilh(kl));
    
    % optimal consumption value
    cli(kl)   = cmat(ill(kl),ilh(kl));
    
    % translate index into the full matrix
    ilh(kl) = (kl-1)+ilh(kl);
    ill(kl) = (kl-1)+ill(kl);

    % recovery values at the optimal choice
    QBlhi(kl) = QBlhmat(ill(kl),ilh(kl));
    QBlli(kl) = QBllmat(ill(kl),ilh(kl));
    
    % continuation values
    EVlhi(kl) = EVlhmat(ill(kl),ilh(kl));
    EVlli(kl) = EVllmat(ill(kl),ilh(kl));

    % work through the remaining lower debt levels
    for kl =(ibhatl_Phi(end)-1):-1:1

    % consumption and momentary utility
    cmat = Psi(kl).*QBlhmat(kl:ill(kl+1),kl:ilh(kl+1))+(1-Psi(kl)).*QBllmat(kl:ill(kl+1),kl:ilh(kl+1))+wl-bl(kl);
    umat = (cmat<=0)*(-10^(6))+(cmat>0).*(u(cmat)-X(Psi(kl)));      % takes care of negative consumption levels
    
    % use knowledge of FOC here
    Wlcond = umat(1,:)+beta*(Psi(kl).*EVlhmat(kl,kl:ilh(kl+1))+(1-Psi(kl)).*EVllmat(kl,kl:ilh(kl+1)));
    ill(kl) = 1;
    % optimal debt, bplh
    bplhi_exact(kl) = ((1-Psi(kl))*QBllmat(kl,1)+(wl-wh)-bl(kl))/(1/R-1-Psi(kl)/R);
    % get index
    ilh(kl)    = min(ilh(kl+1)-(kl-1),sum(bplhi_exact(kl)>=bgrid)-(kl-1));
    Wlnext(kl) = Wlcond(ilh(kl));
    
    % optimal consumption
    cli(kl)   = cmat(ill(kl),ilh(kl));
    
    % translate index into the full matrix
    ilh(kl) = (kl-1)+ilh(kl);
    ill(kl) = (kl-1)+ill(kl);

    % bond revenue at the optimal choice
    QBlhi(kl) = QBlhmat(ill(kl),ilh(kl));
    QBlli(kl) = QBllmat(ill(kl),ilh(kl));
    
    % optimal consumption and continuation values
    EVlhi(kl) = EVlhmat(ill(kl),ilh(kl));
    EVlli(kl) = EVllmat(ill(kl),ilh(kl));

    end

    % fill up the remaining values
    Wlnext(ibhatl_Phi(end)+1:N)  = Wlnext(ibhatl_Phi(end));
    ill(ibhatl_Phi(end)+1:N)     = ill(ibhatl_Phi(end));
    ilh(ibhatl_Phi(end)+1:N)     = ilh(ibhatl_Phi(end));
    QBlhi(ibhatl_Phi(end)+1:N)   = QBlhi(ibhatl_Phi(end));
    QBlli(ibhatl_Phi(end)+1:N)   = QBlli(ibhatl_Phi(end));
    cli(ibhatl_Phi(end)+1:N)     = cli(ibhatl_Phi(end));
    EVlhi(ibhatl_Phi(end)+1:N)   = EVlhi(ibhatl_Phi(end));
    EVlli(ibhatl_Phi(end)+1:N)   = EVlli(ibhatl_Phi(end));
    bplhi_exact(ibhatl_Phi(end)+1:N) = bplhi_exact(ibhatl_Phi(end));

    QBhhat = @(bp) 1/R*(bp.*(1-Fh(bp))+Ih(bp)*(pi.*bhath));
    QBlhat = @(bp) 1/R*(bp.*(1-Fl(bp))+Il(bp)*(pi.*bhatl));

    % update the value function
    Wlfun = @(bp) fnval(bp,fnxtr(csaps(bl(rangel),Wlnext(rangel)),2));

    %%% trick end %%%

    % Howard's improvement (policy function iterations)
    steps = iter*1;              % choose number of iterations

    % fix policy functions
    bphhi = bphh(ihh);
    bphli = bphl(ihl);
    bplhi = bplh(ilh);
    bplli = bpll(ill);

    for t=1:steps
        
        % varies along the row (or along bphh)
        EVhhi = (1-Fh(bphhi)).*Whfun(bphhi)+Ih(bphhi)*(pi.*Whfun(bhath));
        
        % varies along the column (or along bphl)
        EVhli = (1-Fl(bphli)).*Wlfun(bphli)+Il(bphli)*(pi.*Wlfun(bhatl));
        
        % varies along the row (or along bplh)
        EVlhi = (1-Fh(bplhi)).*Whfun(bplhi)+Ih(bplhi)*(pi.*Whfun(bhath));
        
        % varies along the column (or along bpll)
        EVlli = (1-Fl(bplli)).*Wlfun(bplli)+Il(bplli)*(pi.*Wlfun(bhatl));

        % iterate given the policy function
        Whnext = u(ph.*QBhhi+(1-ph).*QBhli+wh-bh)+beta*(ph.*EVhhi+(1-ph).*EVhli);
        
        % iterate given the policy function
        Wlnext = u(Psi.*QBlhi+(1-Psi).*QBlli+wl-bl)-X(Psi)+beta*(Psi.*EVlhi+(1-Psi).*EVlli);

        % update the value function
        Whfun = @(bp) fnval(bp,fnxtr(csaps(bh(rangeh),Whnext(rangeh)),2));
        
        % update the value function
        Wlfun = @(bp) fnval(bp,fnxtr(csaps(bl(rangel),Wlnext(rangel)),2));
    end

    % compute distance
    dist = norm([Wh-Whnext,Wl-Wlnext]);

    % update the value functions
    Wh  = Whnext;
    Wl  = Wlnext;

    % display inner loop, progress
    fprintf('%4i \t %2.1e \n',iter,dist);

    % increase the iteration counter
    iter = iter+1;

    end     % end: inner 
    
    %%% normal times: update bhat_Phi and bhat according to the threshold conditions

    % normal times
    Whhat_Phi = Wh(i0)-phih;
    Whhat     = Wh(i0)-phih;
    
    % interpolate Whhat_Phi on (bh,Wh) to retrieve bhath_Phinext
    bhath_Phinext = fnval(Whhat_Phi,fnxtr(csaps(Wh(rangeh),bh(rangeh)),2));
    
    % interpolate Whhat on (bh,Ch) to retrieve bhathnext
    bhathnext = fnval(Whhat,fnxtr(csaps(Wh(rangeh),bh(rangeh)),2));
                
    % returns the index of the last grid point smaller than bhat_Phinext
    % max make sure the index is in a valid range
    ibhath_Phinext = max(1,sum(repmat(bhath_Phinext,1,N)>=repmat(bgrid',S,1),2));
    bhath_Phinext  = bgrid(ibhath_Phinext);
    ibhathnext     = max(1,sum(repmat(bhathnext,1,N)>=repmat(bgrid',S,1),2));
    bhathnext      = bgrid(ibhathnext);

    %%% recession: update bhat_Phi and bhat according to the threshold conditions

    Wlhat_Phi = Wl(i0)-phil;
    Wlhat     = Wl(i0)-phil;

    % interpolate WChhat_Phi on (bh,WCh) to retrieve bhath_Phinext
    bhatl_Phinext = fnval(Wlhat_Phi,fnxtr(csaps(Wl(rangel),bl(rangel)),2));
    
    % interpolate WChhat on (bh,WCh) to retrieve bhathnext
    bhatlnext = fnval(Wlhat,fnxtr(csaps(Wl(rangel),bl(rangel)),2));

    % returns the index of the last grid point smaller than bhat_Phinext
    % max make sure the index is in a valid range
    ibhatl_Phinext = max(1,sum(repmat(bhatl_Phinext,1,N)>=repmat(bgrid',S,1),2));
    bhatl_Phinext  = bgrid(ibhatl_Phinext);
    ibhatlnext     = max(1,sum(repmat(bhatlnext,1,N)>=repmat(bgrid',S,1),2));
    bhatlnext      = bgrid(ibhatlnext);

    % truncate debt grids according to the new renegotiation
    % threshold
    bhnext = min(bgrid,bhath_Phinext(end));
    blnext = min(bgrid,bhatl_Phinext(end));

    % update the effort function
    Psinext    = max(0,min(1,X1inv(u1(cli)/R.*(QBlhi-QBlli)+beta*(EVlhi-EVlli))));
    
    % compute the convergence criterion
    distout = norm([bhath_Phinext-bhath_Phi;bhathnext-bhath;...
                    bhatl_Phinext-bhatl_Phi;bhatlnext-bhatl;...
                    Whnext-Wh;...
                    Wlnext-Wl;...
                    Psinext-Psi]);
                
    % update all objects
    Wh         = Whnext;
    bhath_Phi  = bhath_Phinext;
    ibhath_Phi = ibhath_Phinext;
    bhath      = bhathnext;
    ibhath     = ibhathnext;
    bh         = bhnext;
    bphh       = bh;
    bphl       = bl;

    % update non-flat range for the interpolation
    rangeh = 1:ibhath_Phi(end);
                
    % update all objects
    Wl         = Wlnext; 
    bhatl_Phi  = bhatl_Phinext;
    ibhatl_Phi = ibhatl_Phinext;
    bhatl      = bhatlnext;
    ibhatl     = ibhatlnext;
    bl         = blnext;
    bplh       = bh;
    bpll       = bl;
    lax        = 0.1;                       % relaxation parameter
    Psi        = (1-lax)*Psi+lax*Psinext;

    % update non-flat range for the interpolation
    rangel = 1:ibhatl_Phi(end);

    % display outer loop, progress
    fprintf('Outer loop: \n');
    fprintf('%4i \t %2.1e \n',iterout,distout);

    iterout = iterout+1;

end

fprintf('Done with the stationary equilibrium. \n');

%% 3. compute expected values given initial debt and save the solution

% compute recovery value for given debt after the realization of w and
% before the realization of the default cost.

Pih_SB = R*QBhhat(bh);
Pil_SB = R*QBlhat(bl);

% expected utility for given debt after the realization of w and
% before the realization of the default cost.

EVh_SB = (1-Fh(bh)).*Whfun(bh)+Ih(bh)*(pi.*Whfun(bhath));
EVl_SB = (1-Fl(bl)).*Wlfun(bl)+Il(bl)*(pi.*Wlfun(bhatl));

% consumption in normal times
chi = ph.*QBhhi+(1-ph).*QBhli+wh-bh;

% save solution
xh = [bphhi,bphli,Wh,chi];
xl = [bplhi,bplli,Wl,cli,Psi];

% uncomment of you want to overwrite the saved solution
% save(strcat('solution_gamma',int2str(gamma)),'bgrid','i0','bh','bl','xh','xl','Psi','bhath_Phi','ibhath_Phi','bhath','ibhath','bhatl_Phi','ibhatl_Phi','bhatl','ibhatl',...
%                                                 'beta','ph','phil','phih','pi','R','wh','wl','u','uinv','u1','u1inv','X','X1','X11','X1inv',...    
%                                                 'Pih_SB','Pil_SB','EVh_SB','EVl_SB');
                                              