% solves MSZ (2019):
% - planner allocation w/o moral hazard in effort
%
% note:
% - uses the endogenous grid method

%% 0. housekeeping

clear all
clc

%% 1. setting parameters and functional forms

% risk aversion parameter
gamma = 2;

% load parameters, functional forms, and solution from second-best
% decentralization
load(strcat('../Decentralization/solution_gamma',int2str(gamma)));

% approximation
N = size(bgrid,1);                       % same number of grid points as for the debt grid
S = size(phil,1);                        % number of default cost state realizations

% value functions decentralized equilibrium
Wh = xh(:,3);
Wl = xl(:,3);

% value of default at zero cost realization is the outside option
vhout = Wh(i0);       
vlout = Wl(i0);

% expected default cost realization
Ephil = sum(phil.*pi);

I     = @(pphi) (repmat(pphi,1,S)<repmat(phil',size(pphi,1),1));
% cumulative probability function
F     = @(pphi) 1-I(pphi)*pi;
% cumulative probability weighted function g(phi)
% pphi is the upper integral bound
Sigma = @(g,pphi) (1-I(pphi))*(pi.*g);

%% 2. compute planner allocation for normal times

% set up grid with S grid points
vgridh = linspace(min(Wh),max(Wh),S)';

% promised utility is the grid
omegah = vgridh;

% guess future consumption function given the current state (constant guess)
cph = uinv((1-beta)*omegah);

%%% iterate on the consumption function

crit    = 10^(-12);
dist    = 1;
iter    = 1;
maxiter = 1000;

fprintf('Solve for the planner allocation in normal time: \n');
fprintf('Iterating on the Euler equation \n');
fprintf('iter \t norm \n');

while (iter<=maxiter&&dist>=crit)

    % compute current consumption from Euler equation
    ch = u1inv(u1(cph)*beta*R);

    % compute initial promised-utility from promise-keeping constraint
    vh = u(ch)+beta*omegah;
    
    % use built-in function extrapolation (fnxtr,csaps)
    
    % get cubic spline approximation
    cfnh = fnxtr(csaps(vh,ch),2);

    % update the future consumption function
    % note:
    % fnxtr is is robust to function extrapolation
    cphnext = fnval(omegah,cfnh);
    
    % compute distance
    dist = norm(cphnext-cph);

%     % displayprogress
%     fprintf('%4i \t %2.1e \n',iter,dist);

    % update the consumption function
    cph     = cphnext;

    % increase counter
    iter = iter+1;

end 

% display final result
fprintf('%4i \t %2.1e \n',iter,dist);
fprintf('Done. \n');

%%% iterate on the profit function

% guess profit function
Ph   = (wh-ch)/(1-1/R);

crit    = 10^(-12);
dist    = 1;
iter    = 1;
maxiter = 1000;

fprintf('Iterating on the Bellman equation \n');
fprintf('iter \t norm \n');

while (iter<=maxiter&&dist>=crit)

    % get cubic spline approximation
    Pfnh = fnxtr(csaps(vh,Ph),2);
    
    % compute the continuation profit
    Pph = fnval(omegah,Pfnh);
    
    Phnext = (wh-ch+1/R*Pph);

    % compute distance
    dist = norm(Phnext-Ph);

%     % display progress
%     fprintf('%4i \t %2.1e \n',iter,dist);

    % update the profit function
    Ph = Phnext;

    % increase counter
    iter = iter+1;
end

% display final result
fprintf('%4i \t %2.1e \n',iter,dist);
fprintf('Done. \n');
fprintf('Done with the planner allocation in normal time. \n');

%% 3. solve for the recession allocation (endogenous grid method)

% set up the natural grid
% note: 
% - omegal is falling in phil
% - phil can be interpreted as the threshold cost realization given omegal
vtill = vlout-Sigma(phil,phil)-phil.*(1-F(phil));

% choose grid for omegall
vlgrid  = vtill;

% load existing solution
load(strcat('solution_gamma',int2str(gamma)),'xh','xl','Psi','vh','vl');
omegahl = xh(:,3);
omegahh = xh(:,2);
omegall = xl(:,3);
omegalh = xl(:,2);
ch      = xh(:,1);
cl      = xl(:,1);
cph     = u1inv(u1(ch)/(beta*R));
cpl     = u1inv(u1(cl)/(beta*R));
Ph      = xh(:,4);
Pl      = xl(:,4);

%%% iterate on the reform effort

critout    = 10^(-13);
distout    = 1;
iterout    = 1;
maxiterout = 100;
fprintf('Solve for the planner allocation: \n');

while (iterout<=maxiterout&&distout>=critout)
    
    %%% iterate on the consumption functions

    crit    = 10^(-12);
    dist    = 1;
    iter    = 1;
    maxiter = 1000;

    fprintf('Iterating on the Euler equation \n');
    fprintf('iter \t norm \n');

    while (iter<=maxiter&&dist>=crit)
        
        % compute current consumption from Euler equation
        ch = u1inv(u1(cph)*beta*R);
        cl = u1inv(u1(cpl)*beta*R);
              
        % make sure interpolation is in the non-flat part of vh
        start1   = find(diff(vh)~=0,1,'first');
        start2   = find(diff(ch)~=0,1,'first');
        start3   = find(diff(Ph)~=0,1,'first');
        start    = max([start1,start2,start3]);
        vhflat   = find(abs(diff(vh(start:end)))<10^(-10),1,'first')+(start-1);
        if isempty(vhflat)
            % note that size of vh is bigger than S, because we appended
            % grid points
            vhflat = size(vh,1);
        end
        chflat   = find(abs(diff(ch(start:end)))<10^(-10),1,'first')+(start-1);
        if isempty(chflat)
            % note that size of cl is bigger than S, because we appended
            % grid points
            chflat = size(ch,1);
        end
        Phflat   = find(abs(diff(Ph(start:end)))<10^(-10),1,'first')+(start-1);
        if isempty(Phflat)
            % note that size of cl is bigger than S, because we appended
            % grid points
            Phflat = size(Ph,1);
        end
        corr  = 0;
        maskh = (start:min([vhflat,chflat,Phflat])-corr)';
        
        % get approximation of the inverse consumption function
        cfnhinv = fnxtr(csaps(ch(maskh),vh(maskh)),2);
        
        % update the promised utility
        % note: the planner cannot take the agent further down than the
        % promised utility associated with the highest default cost
        % realization (equiv. to cannot issue more normal times contingent 
        % debt because the sovereign will default in any future state).
        omegalhnext = max(vh(vhflat),fnval(cpl,cfnhinv));   % lower bound for promised utility kicks in

        % put a floor in the recession
        % threshold participation constraint
        phitill = min(phil(end),vlout-(u(cl)-X(Psi)+beta*(Psi.*omegalh+(1-Psi).*omegall)));
        % promise-keeping constraint
        % note: 
        % - negative phitilh means that no participation constraint
        % binds
        vl      = (phitill>=phil(1)).*(vlout-Sigma(phil,phitill)-phitill.*(1-F(phitill)))...
                  +(phitill<phil(1)).*(u(cl)-X(Psi)+beta*(Psi.*omegalh+(1-Psi).*omegall));
           
        % update the consumption 
             
        % get approximation of the consumption function in normal times
        cfnh = fnxtr(csaps(vh(maskh),ch(maskh)),2);
        
        % update the future consumption in normal times
        cphnext = fnval(omegahh,cfnh);
        
        % make sure interpolation is in the non-flat part of vl
        start1   = find(diff(vl)~=0,1,'first');
        start2   = find(diff(cl)~=0,1,'first');
        start3   = find(diff(Pl)~=0,1,'first');
        start    = max([start1,start2,start3]);
        vlflat   = find(abs(diff(vl(start:end)))<10^(-10),1,'first')+(start-1);
        if isempty(vlflat)
            % note that size of vl is bigger than S, because we appended
            % grid points
            vlflat = size(vl,1);
        end
        clflat   = find(abs(diff(cl(start:end)))<10^(-10),1,'first')+(start-1);
        if isempty(clflat)
            % note that size of cl is bigger than S, because we appended
            % grid points
            clflat = size(cl,1);
        end
        Plflat   = find(abs(diff(Pl(start:end)))<10^(-10),1,'first')+(start-1);
        if isempty(Plflat)
            % note that size of cl is bigger than S, because we appended
            % grid points
            Plflat = size(Pl,1);
        end
        corr  = 0;
        maskl = (start:min([vlflat,clflat,Plflat])-corr)';
              
        % get approximation of the consumption function in normal times
        cfnl = fnxtr(csaps(vl(maskl),cl(maskl)),2);
        
        % update the future consumption in normal times
        cplnext = fnval(omegall,cfnl);

        % get approximation of the inverse consumption function
        cfnlinv = fnxtr(csaps(cl(maskl),vl(maskl)),2);
        
        % update the promised utility
        omegahlnext = fnval(cph,cfnlinv);
             
        % compute distance
        dist = norm([cphnext-cph,cplnext-cpl;...
                     omegahlnext-omegahl,omegalhnext-omegalh]);

%         % displayprogress
%         if mod(iter,100)==0
%             fprintf('%4i \t %2.1e \n',iter,dist);
%         end

        % update consumption and promised utilities with relaxation
        % parameter
        lax     = .01;
        cph     = lax*cphnext+(1-lax)*cph;
        omegahl = lax*omegahlnext+(1-lax)*omegahl;
        cpl     = lax*cplnext+(1-lax)*cpl;
        omegalh = lax*omegalhnext+(1-lax)*omegalh;

        % increase counter
        iter = iter+1;

    end

    % display final result
    fprintf('%4i \t %2.1e \n',iter,dist);
    fprintf('Done. \n');
    
    
    %%% iterate on the profit function

    crit    = 10^(-12);
    dist    = 1;
    iter    = 1;
    maxiter = 1000;             % important to have high accuracy with high risk aversion

    fprintf('Iterating on the Bellman equation \n');
    fprintf('iter \t norm \n');
    
    while (iter<=maxiter&&dist>=crit)

        % get spline approximations
        Pfnh = fnxtr(csaps(vh(maskh),Ph(maskh)),2);
        Pfnl = fnxtr(csaps(vl(maskl),Pl(maskl)),2);
        
        % normal times
        Pphh = fnval(omegahh,Pfnh);
        Pphl = fnval(omegahl,Pfnl);
        
        % recession
        Pplh = fnval(omegalh,Pfnh);
        Ppll = fnval(omegall,Pfnl);

        Plnext = (phitill>=phil(1)).*(...
                (wl-cl+1/R*(Psi.*Pplh+(1-Psi).*Ppll)).*(1-F(phitill))...
                +Sigma(wl-cl+1/R*(Psi.*Pplh+(1-Psi).*Ppll),phitill)...
        )...
        +(phitill<phil(1)).*(...
                wl-cl+1/R*(Psi.*Pplh+(1-Psi).*Ppll)...
        );

        % compute distance
        dist = norm(Plnext-Pl);
        
%         % display progress
%         if mod(iter,100)==0
%             fprintf('%4i \t %2.1e \n',iter,dist);
%         end

        % update the profit function with relaxation
        lax = .01;
        Pl = lax*Plnext+(1-lax)*Pl;

        % increase counter
        iter = iter+1;
    end

    % display final result
    fprintf('%4i \t %2.1e \n',iter,dist);
    fprintf('Done. \n');
    
    % compute the reform effort
    Psinext = max(0,min(1,X1inv(1/R*u1(cpl).*(Pplh-Ppll)+beta*(omegalh-omegall))));

    % compute distance
    distout = norm(Psinext-Psi);
    
    % display progress
    fprintf('============= \n Outer loop \n');
    fprintf('%4i \t %2.1e \n ============ \n',iterout,distout);

    % update the reform effort
    % relaxation parameter
    lax     = .1;
    Psi     = lax*Psinext+(1-lax)*Psi;

    % update counter
    iterout = iterout+1;
    
end

fprintf('Done with the planner allocation. \n');

%%

% save the second-best solution
xh = [ch,omegahh,omegahl,Ph,cph];
xl = [cl,omegalh,omegall,Pl,cpl];

% uncomment if you want to overwrite the saved solution
% save(strcat('solution_gamma',int2str(gamma)),'vh','xh','vl','xl','Psi','beta','ph','phih','phih','pi','R','wh','wl',...
%                                                 'u','uinv','u1','u1inv','X','X1','X1inv');
                                            
%% 4. set up interpolation ready functions

% define spline interpolation as the standard
method = 'spline';

% this is fragile!! generally not true, just a trick
phitilh = phitill;

% threshold realization
Phifun = @(vv,h) h.*max(phih(1),min(phih(end),interp1(vh,phitilh,vv,method)))...
            +(1-h).*max(phil(1),min(phil(end),interp1(vl,phitill,vv,method)));
        
% consumption
Cfun = @(vv,h) h.*interp1(vh,ch,vv,method)+(1-h).*interp1(vl,cl,vv,method);

% consumption as a function of phi
C    = @(pphi,h) h.*interp1(phitilh,ch,pphi,method)+(1-h).*interp1(phitill,cl,pphi,method);

% reform effort
Upsfun = @(vv) max(0,min(1,interp1(vl,Psi,vv,method)));

% effort as a function phi
Ups    = @(pphi) interp1(phitill,Psi,pphi,method);

% promised utility, recovery
Vhfun = @(vv,h) h.*interp1(vh,omegahh,vv,method)+(1-h).*interp1(vl,omegalh,vv,method);

% promised utility, recovery as a function of phi
Vh    = @(pphi,h) h.*interp1(phitilh,omegahh,pphi,method)+(1-h).*interp1(phitill,omegalh,pphi,method);

% promised utility, recession
Vlfun = @(vv,h) h.*interp1(vh,omegahl,vv,method)+(1-h).*interp1(vl,omegall,vv,method);

% promised utility, recession as a function of phi
Vl    = @(pphi,h) h.*interp1(phitilh,omegahl,pphi,method)+(1-h).*interp1(phitill,omegall,pphi,method);

%% 5. simulation with manual shocks (figures paper)

b0      = 1*wl;
ib0     = find(bgrid>=b0,1,'first');
ip0     = find(Pl>=Pil_SB(ib0),1,'first');
istar   = find(EVl_SB<=vl(ip0),1,'first');
vstar   = vl(ip0);
bstar   = bgrid(istar)/wl;

% pin down initial condition (zero profit contract, compared to SC)
v0 = vl(ip0);

% figure out the zero profit promise

% note:
% - simulates only non-redundant economies and periods
nperiods = 15;                                      % number of periods to be simulated
npaths   = 4;                                       % number of paths to be simulated
st       = randi([1,S],npaths,nperiods);            % draws from uniform discrete distribution 

% initialize variables

% current promised-utility
vt = zeros(npaths,nperiods);

% promised-utility boom
vht = zeros(npaths,nperiods+1);

% promised-utility recession
vlt = zeros(npaths,nperiods+1);

% set initial conditions
vlt(:,1)   = v0;                 % zero profit value
vht(:,1)   = Vhfun(v0,0);        % irrelevant since we start in recession

% consumption
ct = zeros(npaths,nperiods);

% recovery probability
pt = zeros(npaths,nperiods);

% effort cost
xt = zeros(npaths,nperiods);

% normal times indicator
ht = zeros(npaths,nperiods+1);

% renegotiation indicator
rt = zeros(npaths,nperiods);

% renegotiation threshold
phit = zeros(npaths,nperiods);

%%% this is for the figures only %%%

% manual default shock series
Tf  = 15;    % final period
Tn  = 13;    % period where you get permanently out of the recession
Tr1 = 9;     % first renegotiation in recession
Tr2 = 11;    % second renegotiation in recession

% create 4 artificial series
% 1...no renegotiation and staying in recession forever
% 2...renegotiation1 and staying in recession forever
% 3...renegotiation1+2 and staying in recession forever
% 4...renegotiation1+2 and transition to normal times forever

st(:,1:Tf)    = S;         % maximum shock realization
st(2:end,Tr1) = 25;        % assign all included series the first shock realization 
st(3:end,Tr2) = 5;

% assign the normal times periods to the last series
ht(4,Tn:end)  = 1;

%%% end %%%

for t=1:nperiods
    vt(:,t)    = ht(:,t).*vht(:,t)+(1-ht(:,t)).*vlt(:,t);
    phit(:,t)  = Phifun(vt(:,t),ht(:,t));
    rt(:,t)    = ht(:,t).*(phih(st(:,t))<phit(:,t))+(1-ht(:,t)).*(phil(st(:,t))<phit(:,t));   % renegotiated proof contract
    ct(:,t)    = rt(:,t).*C(phil(st(:,t)),ht(:,t))+(1-rt(:,t)).*Cfun(vt(:,t),ht(:,t));
    pt(:,t)    = ht(:,t)*ph+(1-ht(:,t)).*(rt(:,t).*Ups(phil(st(:,t)))+(1-rt(:,t)).*Upsfun(vt(:,t)));
    xt(:,t)    = ht(:,t)*0+(1-ht(:,t)).*X(pt(:,t));
    vht(:,t+1) = rt(:,t).*Vh(phil(st(:,t)),ht(:,t))+(1-rt(:,t)).*Vhfun(vt(:,t),ht(:,t));
    vlt(:,t+1) = rt(:,t).*Vl(phil(st(:,t)),ht(:,t))+(1-rt(:,t)).*Vlfun(vt(:,t),ht(:,t));
end

%%

%%% figure: 
%%% consumption, effort and expected utility over time (paper)
%%% 2 renegotiations, 1 permanent recovery

scrsz = get(0,'ScreenSize');
% the size of the figure is important such that the strings appear in
% the slides with the appropriate size.
figure('Position',[scrsz(3)*1/6 scrsz(4)*1/9 scrsz(3)*4/9 ...
    scrsz(4)*1/3]);

% panel (a) 
% planner allocation: consumption and effort
h = subplot(1,2,1);
pos    = get(h, 'position');
pos(1) = pos(1)-0.04;
pos(2) = pos(2)+0.02;
set(h,'position', pos);

% setup plot matrices
ctmat          = ct(:,1:Tf);
ptmat          = pt(:,1:Tf);
ptmat(ht(:,1:end-1)==1) = NaN;
vltmat         = vlt(:,2:Tf+1);
vhtmat         = vht(:,2:Tf+1);
Ttmat          = repmat(1:Tf,[npaths,1]);
Trmat          = [Tr1*ones(1,Tf);Tr2*ones(1,Tf);Tn*ones(1,Tf);Tf*ones(1,Tf)];
Trmatlag       = [ones(1,Tf);Trmat(1:npaths-1,1:Tf)];
mask           = find(Ttmat>Trmat);
masklag        = find(Ttmat<Trmatlag);
ctmat(mask)    = NaN;
ctmat(masklag) = NaN;
ptmat(mask)    = NaN;
ptmat(masklag) = NaN;
vltmat(mask)    = NaN;
vltmat(masklag) = NaN;
vhtmat(mask)    = NaN;
vhtmat(masklag) = NaN;

ct_SB     = ct;
ctmat_SB  = ctmat;
pt_SB     = pt;
ptmat_SB  = ptmat;
vht_SB    = vht;
vhtmat_SB = vhtmat;
vlt_SB    = vlt;
vltmat_SB = vltmat;

% uncomment of you want to overwrite saved simulation results
% % save simulation for export
% save('simulation_figure','ct_SB','ctmat_SB','pt_SB','ptmat_SB','vht_SB','vhtmat_SB','vlt_SB','vltmat_SB');

% mark recession with shaded areas
h = fill([1 1 Tn Tn],[.9 .945 .945 .9],.9*[1,1,1],'edgecolor','none');
hold on;

% plot the time paths
[Ax,l1,l2] = plotyy((1:Tf),ctmat,(1:Tf),ptmat);
hold off;

%title('No moral hazard');
xlabel('Time');
ylabel(Ax(1),'Consumption')
ylabel(Ax(2),'Reform Effort')
legend([h l1(1) l2(1)],'Recession period','Consumption','Reform Effort','Location','NorthWest');
set(l1,'color','b','marker','o','linestyle','-','linewidth',1.5);
set(l2,'color',[0,.5,0],'marker','x','linestyle','-','linewidth',1.5);

ylim(Ax(1),[.9 .945]);
ylim(Ax(2),[.06 .09]);
xlim(Ax(1),[1,Tf]);
xlim(Ax(2),[1,Tf]);

set(Ax(1),'YTick',0.9:.01:0.945,'XTick',[1,5,10,13]);
set(Ax(1),'Box','Off');
set(Ax(2),'YTick',.06:.005:0.09);

% jumps consumption
line([Tr1,Tr1],[ct(1,Tr1) ct(2,Tr1)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');
line([Tr2,Tr2],[ct(2,Tr2) ct(3,Tr2)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');
line([Tn,Tn],[ct(3,Tn) ct(4,Tn)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');

% jumps reform effort
line([Tr1,Tr1],[pt(1,Tr1) pt(2,Tr1)],'parent',Ax(2),'Linewidth',1,'Linestyle',':','color',[0,.5,0]);
line([Tr2,Tr2],[pt(2,Tr2) pt(3,Tr2)],'parent',Ax(2),'Linewidth',1,'Linestyle',':','color',[0,.5,0]);

%%

% panel (b) 
% planner allocation: promised utility

% increase the space between subplots a bit
h      = subplot(1,2,2);
pos    = get(h, 'position');
pos(1) = pos(1)+0.00;
pos(2) = pos(2)+0.02;
set(h,'position', pos);

% mark recession with shaded areas
fill([1 1 Tn Tn],[-60.5 -63 -63 -60.5],.9*[1,1,1],'edgecolor','none');
hold on;

% plot the time paths
[Ax,l1,l2] = plotyy((1:Tn),vltmat(:,1:Tn),(1:Tf),vhtmat);
hold off;

%title('No moral hazard');
xlabel('Time');
ylabel(Ax(1),'Promised utility, recession')
ylabel(Ax(2),'Promised utility, recovery')
legend([l1(1) l2(1)],'Recession','Recovery','Location','NorthWest');
set(l1,'color','r','marker','o','linestyle','-','linewidth',1.5);
set(l2,'color','k','marker','d','linestyle','-','linewidth',1.5);

ylim(Ax(1),[-63 -60.5]);
ylim(Ax(2),[-56 -54]);
xlim(Ax(1),[1,Tf]);
xlim(Ax(2),[1,Tf]);

set(Ax(1),'YTick',-63:.5:-60.5,'XTick',[1,5,10,13]);
set(Ax(1),'Box','Off');
set(Ax(2),'YTick',-56:.5:-54);

% jumps promised utility, recession
line([Tr1,Tr1],[vlt(1,Tr1+1) vlt(2,Tr1+1)],'parent',Ax(1),'Linewidth',1.25,'Linestyle',':','color','r');
line([Tr2,Tr2],[vlt(2,Tr2+1) vlt(3,Tr2+1)],'parent',Ax(1),'Linewidth',1.25,'Linestyle',':','color','r');

% jumps promised utility recovery
line([Tr1,Tr1],[vht(1,Tr1+1) vht(2,Tr1+1)],'parent',Ax(2),'Linewidth',1.25,'Linestyle',':','color','k');
line([Tr2,Tr2],[vht(2,Tr2+1) vht(3,Tr2+1)],'parent',Ax(2),'Linewidth',1.25,'Linestyle',':','color','k');
