% solves MSZ (2019):
% - decentralization of the planner allocation with moral hazard
%
% Note:
% - uses spline extrapolation (Matlab's built-in function fnxtr) 
% to evaluate the unkown functions outside and between the grid points.

%% 0. housekeeping
clear all
clc

fprintf('Replicating Figure 3... \n\n');

%% 1. setting parameters

% flag for existing solution
solution = 1;

% preferences
gamma = 2;                      % risk aversion parameter

% parameters, functions, and solution of the one asset economy with the
% regular debt grid
load(strcat('../../One asset with renegotiation/solution_gamma',int2str(gamma)));

% save variables under different names
bgrid_R = bgrid; bh_R = bh; bl_R = bl; xh_R = xh; xl_R = xl; Psi_R = Psi;
ibhatl_Phi_R = ibhatl_Phi; ibhath_Phi_R = ibhath_Phi;

% allows for a coarser debt grid as the curse of dimensionality starts
% kicking in with the choice of two debt levels
N = 1000;

% number of shock states;
S = size(phil,1);

% setup new (coarser) debt grid with the same span
bmax = bh_R(end);
bmin = bh_R(1);

% build up equally spaced grid around 0
i0            = ceil(N/10);           % number of asset grid points
Nb            = N-i0;                 % number of debt grid points
bgrid         = zeros(N,1);
bgrid(1:i0,:) = linspace(bmin,0,i0)';
% i0 will denote the index of the zero debt value
bgrid(i0:end,:) = linspace(0,bmax,Nb+1)';

% specify interpolation method
method = 'spline';

% load solution
load(strcat('solution_gamma',int2str(gamma)),'bh','bl','xl','xh',...
                                             'Psi',...
                                             'bhath_Phi','ibhath_Phi','bhath','ibhath',...
                                             'bhatl_Phi','ibhatl_Phi','bhatl','ibhatl');
% assign the value functions
Wh        = xh(:,3);
Wl        = xl(:,3);

% choose grid for the future debt
% starting in normal times
bph  = bh;
bphh = bh;
bphl = bl;

% starting in recession
bplh = bh;
bpll = bl;       % recession grid is smaller

% define non-flat ranges for interpolation
rangeh = 1:ibhath_Phi(end);
rangel = 1:ibhatl_Phi(end);

%% 2. solve for the stationary equilibrium (value function iteration)

% prepare outer loop
critout    = 10^(-12);
distout    = 1;
iterout    = 1;
maxiterout = 5;

fprintf('Solve for the stationary equilibrium: \n');

while (iterout<=maxiterout)&&(distout>critout)

    % continuation in boom
    % indicates whether bp is bigger or equal than the renegotiation threshold, bhat_Phi
    Ih = @(bp) (repmat(bp,1,S)>=repmat(bhath_Phi',N,1));
    Il = @(bp) (repmat(bp,1,S)>=repmat(bhatl_Phi',N,1));

    % default probability
    Fh  = @(bp) Ih(bp)*pi;
    Fl  = @(bp) Il(bp)*pi;
    
    Whfun = @(bp) fnval(bp,fnxtr(csaps(bh(rangeh),Wh(rangeh)),2));
    Wlfun = @(bp) fnval(bp,fnxtr(csaps(bl(rangel),Wl(rangel)),2));
    
    % general remark: mat variables vary with bpxl long the column and with bpxh along the row
    EVlhmat = repmat(((1-Fh(bplh)).*Whfun(bplh)+Ih(bplh)*(pi.*Whfun(bhath)))',N,1);
    EVllmat = repmat(((1-Fl(bpll)).*Wlfun(bpll)+Il(bpll)*(pi.*Wlfun(bhatl))),1,N);      
    
    % compute recovery probability from the optimality condition
    % note: use existing solution which is more precise than the
    % interpolated one
    lax = 0;
    Psi = Psi*(1-lax)+lax*max(0,min(1,X1inv(beta*(EVlhmat-EVllmat))));

    % recovery value
    QBhhat = @(bp) 1/R*(bp.*(1-Fh(bp))+Ih(bp)*(pi.*bhath));
    QBlhat = @(bp) 1/R*(bp.*(1-Fl(bp))+Il(bp)*(pi.*bhatl));

    % matrix form allowing to search over all possible combinations of future
    % debt accumulation
    QBhhmat = ph.*repmat(QBhhat(bphh)',N,1);       % varies with bpxh along rows
    QBhlmat = (1-ph).*repmat(QBlhat(bphl),1,N);    % varies with bpxl along columns

    QBlhmat = Psi.*repmat(QBhhat(bplh)',N,1);       % varies with bpxh along rows
    QBllmat = (1-Psi).*repmat(QBlhat(bpll),1,N);    % varies with bpxl along columns

    % prepare inner loop
    crit    = 10^(-12);                 % convergence criterion
    dist    = 1;                        % norm of the step
    iter    = 2;                        % iteration counter
    maxiter = 50;                       % maximum iterations

    % display inner loop, header
    fprintf('inner loop: \n');
    fprintf('iter \t norm \n');

    while (dist>crit)&&(iter<=maxiter) 

    %%% trick start %%%

    % uses the property that the debt accumulation function will be
    % (weakly) increasing in initial debt
    % speeds up the code substantially when working with large N

    % initialize variables
    
    % normal times
    Whnext = zeros(N,1);
    ihh     = zeros(N,1);
    ihl     = zeros(N,1);
    QBhhi   = ones(N,1);
    QBhli   = ones(N,1);
    
    % recession
    Wlnext = zeros(N,1);
    ilh     = zeros(N,1);
    ill     = zeros(N,1);
    QBlhi   = ones(N,1);
    QBlli   = ones(N,1);
    Psii    = zeros(N,1);

    %%% normal times

    % continuation values

    % varies along the row (or along bpxh)
    EVhhmat = repmat(...
                ((1-Fh(bphh)).*Whfun(bphh)+Ih(bphh)*(pi.*Whfun(bhath)))'...
            ,N,1);

    % varies along the column (or along bpxl)
    EVhlmat = repmat(...
                (1-Fl(bphl)).*Wlfun(bphl)+Il(bphl)*(pi.*Wlfun(bhatl))...
            ,1,N);

    % start with the highest initial debt
    kh = ibhath_Phi(end);

    % consumption
    cmat = QBhhmat+QBhlmat+wh-bh(kh);

    % momentary utility
    umat = (cmat<=0)*(-10^(6))+(cmat>0).*u(cmat); 

    % search for the max first along the row (along bphh), and then along the
    % column (along bphl)
    [Whcond,ihhcond]     = max(umat(rangel,rangeh)+beta*(ph*EVhhmat(rangel,rangeh))+(1-ph)*EVhlmat(rangel,rangeh),[],2);
    [Whnext(kh),ihl(kh)] = max(Whcond,[],1);
    ihh(kh)               = ihhcond(ihl(kh));

    % bond revenue at the optimal choice
    QBhhi(kh) = QBhhmat(ihl(kh),ihh(kh));
    QBhli(kh) = QBhlmat(ihl(kh),ihh(kh));

    % work through the remaining lower debt levels
    for kh =(ibhath_Phi(end)-1):-1:1

    % consumption and momentary utility
    cmat   = QBhhmat+QBhlmat+wh-bh(kh);
    umat   = (cmat<=0)*(-10^(6))+(cmat>0).*u(cmat);      % takes care of negative consumption levels

    % search for the max first along the row (along bphh), and then along the
    % column (along bphl)
    [Whcond,ihhcond]     = max(umat(rangel,rangeh)+beta*(ph*EVhhmat(rangel,rangeh)+(1-ph)*EVhlmat(rangel,rangeh)),[],2);
    [Whnext(kh),ihl(kh)] = max(Whcond,[],1);
    ihh(kh)               = ihhcond(ihl(kh));

    % bond revenue at the optimal choice
    QBhhi(kh) = QBhhmat(ihl(kh),ihh(kh));
    QBhli(kh) = QBhlmat(ihl(kh),ihh(kh));

    end

    % fill up the remaining values
    Whnext(ibhath_Phi(end)+1:end) = Whnext(ibhath_Phi(end));
    ihl(ibhath_Phi(end)+1:end)     = ihl(ibhath_Phi(end));
    ihh(ibhath_Phi(end)+1:end)     = ihh(ibhath_Phi(end));
    QBhhi(ibhath_Phi(end)+1:end)   = QBhhi(ibhath_Phi(end));
    QBhli(ibhath_Phi(end)+1:end)   = QBhli(ibhath_Phi(end));

    % update the value function
    Whfun = @(bp) fnval(bp,fnxtr(csaps(bh(rangeh),Whnext(rangeh)),2));
    
    %%% recession
    
    % varies along the row (or along bplh)
    EVlhmat = repmat(...
                ((1-Fh(bplh)).*Whfun(bplh)+Ih(bplh)*(pi.*Whfun(bhath)))'...
            ,N,1);

    % varies along the column (or along bpll)
    EVllmat = repmat(...
                (1-Fl(bpll)).*Wlfun(bpll)+Il(bpll)*(pi.*Wlfun(bhatl))...
            ,1,N);

    % start with the highest initial debt
    kl = ibhatl_Phi(end);

    % consumption
    cmat   = QBlhmat(kl:rangel(end),kl:rangeh(end))+QBllmat(kl:rangel(end),kl:rangeh(end))+wl-bl(kl);  

    % momentary utility
    umat   = (cmat<=0)*(-10^(6))+(cmat>0).*(u(cmat)-X(Psi(kl:rangel(end),kl:rangeh(end))));      % takes care of negative consumption levels

    % search for the max first along the row (along bphh), and then along the
    % column (along bphl)
    [Wlcond,ilhcond]     = max(umat...
                            +beta*(Psi(kl:rangel(end),kl:rangeh(end)).*EVlhmat(kl:rangel(end),kl:rangeh(end))+(1-Psi(kl:rangel(end),kl:rangeh(end))).*EVllmat(kl:rangel(end),kl:rangeh(end))),[],2);
    [Wlnext(kl),ill(kl)] = max(Wlcond,[],1);
    ilh(kl)               = ilhcond(ill(kl));
    
    ilh(kl) = (kl-1)+ilh(kl);
    ill(kl) = (kl-1)+ill(kl);

    % bond revenue at the optimal choice
    QBlhi(kl) = QBlhmat(ill(kl),ilh(kl));
    QBlli(kl) = QBllmat(ill(kl),ilh(kl));
    Psii(kl)  = Psi(ill(kl),ilh(kl));

    % work through the remaining lower debt levels
    for kl =(ibhatl_Phi(end)-1):-1:1

    % consumption and momentary utility
    cmat   = QBlhmat(kl:ill(kl+1),kl:ilh(kl+1))+QBllmat(kl:ill(kl+1),kl:ilh(kl+1))+wl-bl(kl);
    umat   = (cmat<=0)*(-10^(6))+(cmat>0).*(u(cmat)-X(Psi(kl:ill(kl+1),kl:ilh(kl+1))));      % takes care of negative consumption levels

    % search for the max first along the row (along bplh), and then along the
    % column (along bpll)
    [Wlcond,ilhcond]     = max(umat...
                            +beta*(Psi(kl:ill(kl+1),kl:ilh(kl+1)).*EVlhmat(kl:ill(kl+1),kl:ilh(kl+1))+(1-Psi(kl:ill(kl+1),kl:ilh(kl+1))).*EVllmat(kl:ill(kl+1),kl:ilh(kl+1))),[],2);
    [Wlnext(kl),ill(kl)] = max(Wlcond,[],1);
    ilh(kl)               = ilhcond(ill(kl));
    
    ilh(kl) = (kl-1)+ilh(kl);
    ill(kl) = (kl-1)+ill(kl);

    % bond revenue at the optimal choice
    QBlhi(kl) = QBlhmat(ill(kl),ilh(kl));
    QBlli(kl) = QBllmat(ill(kl),ilh(kl));
    Psii(kl)  = Psi(ill(kl),ilh(kl));

    end

    % fill up the remaining values
    Wlnext(ibhatl_Phi(end)+1:N) = Wlnext(ibhatl_Phi(end));
    ill(ibhatl_Phi(end)+1:N)     = ill(ibhatl_Phi(end));
    ilh(ibhatl_Phi(end)+1:N)     = ilh(ibhatl_Phi(end));
    QBlhi(ibhatl_Phi(end)+1:N)   = QBlhi(ibhatl_Phi(end));
    QBlli(ibhatl_Phi(end)+1:N)   = QBlli(ibhatl_Phi(end));
    Psii(ibhatl_Phi(end)+1:N)    = Psii(ibhatl_Phi(end));

    % update the value function
    Wlfun = @(bp) fnval(bp,fnxtr(csaps(bl(rangel),Wlnext(rangel)),2));

    %%% trick end %%%

    % Howard's improvement (policy function iterations)
    steps = iter*0;              % choose number of iterations

    % fix policy functions
    bphhi = bphh(ihh);
    bphli = bphl(ihl);
    bplhi = bplh(ilh);
    bplli = bpll(ill);

    for t=1:steps
        
        % varies along the row (or along bphh)
        EVhhi = (1-Fh(bphhi)).*Whfun(bphhi)+Ih(bphhi)*(pi.*Whfun(bhath));
        
        % varies along the column (or along bphl)
        EVhli = (1-Fl(bphli)).*Wlfun(bphli)+Il(bphli)*(pi.*Wlfun(bhatl));
        
        % varies along the row (or along bplh)
        EVlhi = (1-Fh(bplhi)).*Whfun(bplhi)+Ih(bplhi)*(pi.*Whfun(bhath));
        
        % varies along the column (or along bpll)
        EVlli = (1-Fl(bplli)).*Wlfun(bplli)+Il(bplli)*(pi.*Wlfun(bhatl));

        % iterate given the policy function
        Whnext = u(QBhhi+QBhli+wh-bh)+beta*(ph.*EVhhi+(1-ph).*EVhli);
        
        % iterate given the policy function
        Wlnext = u(QBlhi+QBlli+wl-bl)-X(Psii)+beta*(Psii.*EVlhi+(1-Psii).*EVlli);

        % update the value function
        Whfun = @(bp) fnval(bp,fnxtr(csaps(bh(rangeh),Whnext(rangeh)),2));
        
        % update the value function
        Wlfun = @(bp) fnval(bp,fnxtr(csaps(bl(rangel),Wlnext(rangel)),2));
    end

    % compute distance
    dist = norm([Wh-Whnext,Wl-Wlnext]);

    % update the value functions
    Wh  = Whnext;
    Wl  = Wlnext;

    % display inner loop, progress
    fprintf('%4i \t %2.1e \n',iter,dist);

    % increase the iteration counter
    iter = iter+1;

    end     % end: inner 
    
    %%% normal times: update bhat_Phi and bhat according to the threshold conditions

    % normal times
    Whhat_Phi = Wh(i0)-phih;
    Whhat     = Wh(i0)-phih;

    % interpolate WChhat_Phi on (bh,WCh) to retrieve bhath_Phinext
    bhath_Phinext = fnval(Whhat_Phi,fnxtr(csaps(Wh(rangeh),bh(rangeh)),2));
    
    % interpolate WChhat on (bh,WCh) to retrieve bhathnext
    bhathnext = fnval(Whhat,fnxtr(csaps(Wh(rangeh),bh(rangeh)),2));
                
    % returns the index of the last grid point smaller than bhat_Phinext
    % max make sure the index is in a valid range
    ibhath_Phinext = max(1,sum(repmat(bhath_Phinext,1,N)>=repmat(bgrid',S,1),2));
    bhath_Phinext  = bgrid(ibhath_Phinext);
    ibhathnext     = max(1,sum(repmat(bhathnext,1,N)>=repmat(bgrid',S,1),2));
    bhathnext      = bgrid(ibhathnext);

    %%% recession: update bhat_Phi and bhat according to the threshold conditions

    Wlhat_Phi = Wl(i0)-phil;
    Wlhat     = Wl(i0)-phil;

    % interpolate WChhat_Phi on (bh,WCh) to retrieve bhath_Phinext
    bhatl_Phinext = fnval(Wlhat_Phi,fnxtr(csaps(Wl(rangel),bl(rangel)),2));
    
    % interpolate WChhat on (bh,WCh) to retrieve bhathnext
    bhatlnext = fnval(Wlhat,fnxtr(csaps(Wl(rangel),bl(rangel)),2));

    % returns the index of the last grid point smaller than bhat_Phinext
    % max make sure the index is in a valid range
    ibhatl_Phinext = max(1,sum(repmat(bhatl_Phinext,1,N)>=repmat(bgrid',S,1),2));
    bhatl_Phinext  = bgrid(ibhatl_Phinext);
    ibhatlnext     = max(1,sum(repmat(bhatlnext,1,N)>=repmat(bgrid',S,1),2));
    bhatlnext      = bgrid(ibhatlnext);

    % truncate debt grids according to the new renegotiation
    % threshold
    bhnext = min(bgrid,bhath_Phinext(end));
    blnext = min(bgrid,bhatl_Phinext(end));
  
    % compute the convergence criterion
    distout = norm([bhath_Phinext-bhath_Phi;bhathnext-bhath;...
                    bhatl_Phinext-bhatl_Phi;bhatlnext-bhatl;...
                    Whnext-Wh;...
                    Wlnext-Wl]);
                
    % update all objects
    Wh         = Whnext;
    bhath_Phi  = bhath_Phinext;
    ibhath_Phi = ibhath_Phinext;
    bhath      = bhathnext;
    ibhath     = ibhathnext;
    bh         = bhnext;
    bphh       = bh;
    bphl       = bl;

    % update non-flat range for the interpolation
    rangeh = 1:ibhath_Phi(end);
                
    % update all objects
    Wl         = Wlnext; 
    bhatl_Phi  = bhatl_Phinext;
    ibhatl_Phi = ibhatl_Phinext;
    bhatl      = bhatlnext;
    ibhatl     = ibhatlnext;
    bl         = blnext;
    bplh       = bh;
    bpll       = bl;

    % update non-flat range for the interpolation
    rangel = 1:ibhatl_Phi(end);

    % display outer loop, progress
    fprintf('Outer loop: \n');
    fprintf('%4i \t %2.1e \n',iterout,distout);

    iterout = iterout+1;

end

fprintf('Done with the stationary equilibrium. \n');

%% 3. compute expected values given initial debt and save the solution

% compute recovery value for given debt after the realization of w and
% before the realization of the default cost.

Pih_SC = R*QBhhat(bh);
Pil_SC = R*QBlhat(bl);

% expected utility for given debt after the realization of w and
% before the realization of the default cost.

EVh_SC = (1-Fh(bh)).*Whfun(bh)+Ih(bh)*(pi.*Whfun(bhath));
EVl_SC = (1-Fl(bl)).*Wlfun(bl)+Il(bl)*(pi.*Wlfun(bhatl));

% consumption
chi = QBhhi+QBhli+wh-bh;
cli = QBlhi+QBlli+wl-bl;

% save solution
xh = [bphhi,bphli,Wh,chi];
xl = [bplhi,bplli,Wl,cli,Psii];

% uncomment if you want to overwrite the saved solution
% save(strcat('solution_gamma',int2str(gamma)),'bgrid','i0','bh','bl','xh','xl','Psi','bhath_Phi','ibhath_Phi','bhath','ibhath','bhatl_Phi','ibhatl_Phi','bhatl','ibhatl',...
%                                                 'beta','ph','phil','phih','pi','R','wh','wl','u','uinv','u1','u1inv','X','X1','X11','X1inv',...    
%                                                 'Pih_SC','Pil_SC','EVh_SC','EVl_SC');
%     
  
%% 4. plot policy functions (decentralized equilibrium)

% figure out where the kink in bplli is
kink   = find(bplli>=bplli(end),1,'first');
blkink = bl(kink);

%% comparison, debt accumulation and effort: Two assets economy vs. One asset economy with renegotiation 

% load one asset economy policy functions
load('../../One asset with renegotiation/policy_functions','bl_R','bpli_R','cl_R','Psii_R','bhatl_R','bhath_R','Fl_R','Fh_R');

% identify discontinuities
dis       = diff(bpli_R);
% identifies 3 significant discontinuities > .015
dis_index = find(dis>.015);

scrsz = get(0,'ScreenSize');
figure('Position',[scrsz(3)*1/8 scrsz(4)*1/8 scrsz(3)*1/2 ...
    scrsz(4)*3/10]);

% debt accumulation
subplot(1,2,1)
hh = plot(bl,bplli,bl(1:150:end),bplhi(1:150:end),bl,bl,bl_R,bpli_R);
set(hh(1),'Color','r','LineWidth',1.5);
set(hh(2),'Color','b','LineWidth',1.5,'Marker','o','MarkerSize',4);
set(hh(3),'Color','k','LineWidth',1','LineStyle','-.');
set(hh(4),'Color',.5*[0,1,0],'LineWidth',1.5,'LineStyle','--');
hh(5) = line([bhatl_Phi(end),bhatl_Phi(end)],[0,max(bplhi)],...
        'LineWidth',1.25,'color','k','linestyle',':');
hh(6) = line([blkink,blkink],[min(bl),max(bplhi)],...
         'LineWidth',1.25,'color','k','linestyle',':');
%%% start: discontinuities
hold on;
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [bpli_R(dis_index(1)),bpli_R(dis_index(1)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [bpli_R(dis_index(1)),bpli_R(dis_index(1)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [bpli_R(dis_index(2)),bpli_R(dis_index(2)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [bpli_R(dis_index(2)),bpli_R(dis_index(2)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [bpli_R(dis_index(3)),bpli_R(dis_index(3)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [bpli_R(dis_index(3)),bpli_R(dis_index(3)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
hold off;
%%% end: discontinuities
title('Debt accumulation');
xlim([0 bhatl_Phi(end)+.05]);
ylim([0-.01,max(bplhi)+.01])
xlabel('Current debt, b'); ylabel('$B(b),\bar{B}(b)$','interpreter','latex');
legend([hh(1) hh(2) hh(4) hh(3)],{'Recession-cont.','Recovery-cont.','Noncontingent',...
                    '$45^{\circ}$-line'},...
                    'interpreter','latex','Location','NorthWest');
                
set(gca,'XTick',[0,0.3,0.6,0.9,blkink-.01,bhatl_Phi(end)+.05]);
xticks        = get(gca,'XTickLabel');
xticks{end-1} = 'b^{+}';
xticks{end}   = 'b^{max}';
set(gca,'XTickLabel',xticks);

% reform effort
subplot(1,2,2)

% smoothen plot
PPsii = fnval(bl,fnxtr(csaps(bl,Psii,.999999),2));

hh = plot(bl,PPsii,bl_R,Psii_R);
set(hh(1),'Color','r','LineWidth',1.5);
set(hh(2),'Color',.5*[0,1,0],'LineWidth',1.5,'LineStyle','--');
line([bhatl(end) bhatl(end)],[Psii(end)-.002 max(Psii_R)+.05],'Linewidth',1.25,'Linestyle',':','Color','k');
line([blkink,blkink],[Psii(end)-.002,max(Psii_R)+.002],...
         'LineWidth',1.25,'color','k','linestyle',':');
%%% start: discontinuities
hold on;
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [Psii_R(dis_index(1)),Psii_R(dis_index(1)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(1)),bl_R(dis_index(1))],...
      [Psii_R(dis_index(1)),Psii_R(dis_index(1)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [Psii_R(dis_index(2)),Psii_R(dis_index(2)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(2)),bl_R(dis_index(2))],...
      [Psii_R(dis_index(2)),Psii_R(dis_index(2)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [Psii_R(dis_index(3)),Psii_R(dis_index(3)+1)],...
      'linewidth',1.5,'Color','white');
line([bl_R(dis_index(3)),bl_R(dis_index(3))],...
      [Psii_R(dis_index(3)),Psii_R(dis_index(3)+1)],...
      'color',.5*[0,1,0],'LineWidth',1,'LineStyle',':');
hold off;
%%% end: discontinuities
title('Reform effort')
ylabel('$\Psi(B(b),\bar{B}(b))$','interpreter','latex');
xlabel('Current debt, b')
xlim([0,max(bl)+.05]);
ylim([Psii(end)-.002,max(Psii_R)+.002]);

set(gca,'XTick',[0,0.3,0.6,0.9,blkink-.01,bhatl_Phi(end)+.05]);
xticks        = get(gca,'XTickLabel');
xticks{end-1} = 'b^{+}';
xticks{end}   = 'b^{max}';
set(gca,'XTickLabel',xticks);
