% solves MSZ (2019):
% - planner allocation with moral hazard (ex-post formulation)
%
% Note:
% - uses spline extrapolation (Matlab's built-in function fnxtr) 
% to evaluate the unkown functions outside and between the grid points.

%% 0. housekeeping
clear all
clc

fprintf('Replicating Figures 1 and 2... \n\n');

%% 1. setting parameters

% risk aversion parameter
gamma = 2;

% load parameters, functional forms, and solution from decentralization
% with moral hazard
load(strcat('../Decentralization/solution_gamma',int2str(gamma)));

% approximation
N = size(bgrid,1);         % number of grid points on the debt grid
S = size(phil,1);          % number of default cost state realizations  

% value functions decentralized equilibrium
Wh = xh(:,3);
Wl = xl(:,3);

% value of default
vhaut = Wh(i0);       
vlaut = Wl(i0); 

% expected default cost realization
Ephil = sum(phil.*pi);

I     = @(pphi) (repmat(pphi,1,S)<repmat(phil',size(pphi,1),1));
% cumulative probability function
F     = @(pphi) 1-I(pphi)*pi;
% cumulative probability weighted function g(phi)
% pphi is the upper integral bound
Sigma = @(g,pphi) (1-I(pphi))*(pi.*g);

%% 2. solve for planner allocation in normal time

% set up grid with N grid points
vgridh = linspace(min(Wh),max(Wh),N)';

% promised utility is the grid
omegah = vgridh;

% guess future consumption function given the current state (constant guess)
cph = uinv((1-beta)*omegah);

%%% iterate on the consumption function

crit    = 10^(-12);
dist    = 1;
iter    = 1;
maxiter = 1000;

fprintf('Solve for planner allocation in normal time: \n');
fprintf('Iterating on the Euler equation \n');
fprintf('iter \t norm \n');

while (iter<=maxiter&&dist>=crit)

    % compute current consumption from Euler equation
    ch = u1inv(u1(cph)*beta*R);

    % compute initial promised-utility from promise-keeping constraint
    vh = u(ch)+beta*omegah;
    
    % use built-in function extrapolation (fnxtr,csaps)
    
    % get cubic spline approximation
    cfnh = fnxtr(csaps(vh,ch),2);

    % update the future consumption function
    % note:
    % fnxtr is is robust to function extrapolation
    cphnext = fnval(omegah,cfnh);
    
    % compute distance
    dist = norm(cphnext-cph);

%     % displayprogress
%     fprintf('%4i \t %2.1e \n',iter,dist);

    % update the consumption function
    cph     = cphnext;

    % increase counter
    iter = iter+1;

end 

% display final result
fprintf('%4i \t %2.1e \n',iter,dist);
fprintf('Done. \n');

%%% iterate on the profit function

% guess profit function
Ph   = (wh-ch)/(1-1/R);

crit    = 10^(-12);
dist    = 1;
iter    = 1;
maxiter = 1000;

fprintf('Iterating on the Bellman equation \n');
fprintf('iter \t norm \n');

while (iter<=maxiter&&dist>=crit)

    % get cubic spline approximation
    Pfnh = fnxtr(csaps(vh,Ph),2);
    
    % compute the continuation profit
    Pph = fnval(omegah,Pfnh);
    
    Phnext = (wh-ch+1/R*Pph);

    % compute distance
    dist = norm(Phnext-Ph);

%     % display progress
%     fprintf('%4i \t %2.1e \n',iter,dist);

    % update the profit function
    Ph = Phnext;

    % increase counter
    iter = iter+1;
end

% display final result
fprintf('%4i \t %2.1e \n',iter,dist);
fprintf('Done. \n');
fprintf('Done with planner allocation in normal time. \n');

%% 3. solve for planner allocation (value function iteration)

% load existing solution
load(strcat('solution_gamma',int2str(gamma)),'vh','vl','xh','xl');

% assign the profit functions
Ph = xh(:,3);
Pl = xl(:,3);

% % approximate with a cubic spline
% Ph_spline = fnxtr(csaps(vh,Ph),2);
% Pl_spline = fnxtr(csaps(vl,Pl),2);

% use min- and max+ of the natural state space as the bounds
% note:
% - v is now the ex-post (after the realization of phi) value of the agent
% - with v>vhaut there is never recontracting (corresponds to b<0)
% - with v<vhaut-max(phi) there is always recontracting (corresponds to b>bar(b))
% - usees equally-spaced grid points
% - initial utility is decreasing on the grid (corresponds to increasing debt)
vl = -linspace(-(vlaut),-(vlaut-max(phil)),N)';

% threshold function is known!
% note:
% - since we extended the grid, this function will become negative for high
% promised utilities, v
% - this does not cause a problem for the solution procedure
Phih  = vhaut-vh;
Phil  = vlaut-vl;

% associated recontracting value as well!
vhath = vhaut-phih;
vhatl = vlaut-phil;

% set up grid for promised-utility
vphh = vh;
vplh = vh;
vphl = vl;
vpll = vl;

% prepare loop
crit    = 10^(-12);                 % convergence criterion
dist    = 1;                        % norm of the step
iter    = 1;                        % iteration counter
maxiter = 100;                       % maximum iterations

fprintf('Solve for the planner allocation:\n');

while (dist>crit)&&(iter<=maxiter)
    
    % indicates whether vp is smaller or equal than the renegotiation threshold, vhat
    Ih = @(vvp) (repmat(vvp,1,S)<=repmat(vhath',N,1));
    Il = @(vvp) (repmat(vvp,1,S)<=repmat(vhatl',N,1));

    % default probability
    Fh = @(vvp) Ih(vvp)*pi;
    Fl = @(vvp) Il(vvp)*pi;

    % set up extrapolation ready profit functions

    % get cubic spline
    Ph_spline = fnxtr(csaps(vh,Ph),2);
    Pl_spline = fnxtr(csaps(vl,Pl),2);

    % set up function
    Phfun     = @(vvp) fnval(vvp,Ph_spline);
    Plfun     = @(vvp) fnval(vvp,Pl_spline);
    
    % continuation profits normal times
    EPhhmat = repmat(((1-Fh(vphh)).*Phfun(vphh)+Ih(vphh)*(pi.*Phfun(vhath)))',N,1);
    EPhlmat = repmat(((1-Fl(vphl)).*Plfun(vphl)+Il(vphl)*(pi.*Plfun(vhatl))),1,N);
    
    % continuation values recession
    Evphhmat = repmat(((1-Fh(vphh)).*vphh+Ih(vphh)*(pi.*vhath))',N,1);
    Evphlmat = repmat(((1-Fl(vphl)).*vphl+Il(vphl)*(pi.*vhatl)),1,N);

    % start looping over the normal times grid
    
    % initialize variables    
    Phnext = zeros(N,1);
    chi    = zeros(N,1);
    ihh    = zeros(N,1);
    ihl    = zeros(N,1);
    Evphli = zeros(N,1);
    Evphhi = zeros(N,1);
    
    for kh=1:N
    
    % consumption    
    chmat = uinv(vhaut-Phih(kh)-beta*(ph*Evphhmat+(1-ph)*Evphlmat));

    % clean up negative consumption values
    % makes sure they will never be chosen
    chmat(chmat<=0) = 10^(9);

    % search for the max first along the row (along vpxh) and then along the column (along vpxl)
    [Phcond,ihhcond]     = max(wh-chmat+1/R*(ph*EPhhmat+(1-ph)*EPhlmat),[],2);
    [Phnext(kh),ihl(kh)] = max(Phcond,[],1);
    ihh(kh)              = ihhcond(ihl(kh));
    
    % optimal consumption
    chi(kh) = chmat(ihl(kh),ihh(kh));
    
    % optimal continuation value
    Evphli(kh) = Evphlmat(ihl(kh),ihh(kh));
    Evphhi(kh) = Evphhmat(ihl(kh),ihh(kh));
    
    end

    % update the profit function

    % get cubic spline
    Ph_spline = fnxtr(csaps(vh,Phnext),2);

    % set up function
    Phfun     = @(vvp) fnval(vvp,Ph_spline);

    % continuation profits recession
    EPlhmat = repmat(((1-Fh(vplh)).*Phfun(vplh)+Ih(vplh)*(pi.*Phfun(vhath)))',N,1);
    EPllmat = repmat(((1-Fl(vpll)).*Plfun(vpll)+Il(vpll)*(pi.*Plfun(vhatl))),1,N);
    
    % continuation values recession
    Evplhmat = repmat(((1-Fh(vplh)).*vplh+Ih(vplh)*(pi.*vhath))',N,1);
    Evpllmat = repmat(((1-Fl(vpll)).*vpll+Il(vpll)*(pi.*vhatl)),1,N);
    
    % effort
    Upsmat  = max(0,min(1,X1inv(beta*(Evplhmat-Evpllmat))));

    % start looping over the recession grid
    
    % initialize variables    
    Plnext = zeros(N,1);
    cli    = zeros(N,1);
    Upsi   = zeros(N,1);
    ilh    = zeros(N,1);
    ill    = zeros(N,1);
    Evplli = zeros(N,1);
    Evplhi = zeros(N,1);
    objective = zeros(N,N,N);
    
    for kl=1:N
    
    % consumption    
    clmat = uinv(vlaut-Phil(kl)-(-X(Upsmat)+beta*(Upsmat.*Evplhmat+(1-Upsmat).*Evpllmat)));

    % clean up negative consumption values
    % makes sure they will never be chosenand
    clmat(clmat<=0) = 10^(9);

    % search for the max first along the row (along vpxh) and then along the column (along vpxl)
    objective(:,:,kl)   = wl-clmat+1/R*(Upsmat.*EPlhmat+(1-Upsmat).*EPllmat);
    [Plcond,ilhcond]     = max(wl-clmat+1/R*(Upsmat.*EPlhmat+(1-Upsmat).*EPllmat),[],2);
    [Plnext(kl),ill(kl)] = max(Plcond,[],1);
    ilh(kl)              = ilhcond(ill(kl));
    
    % optimal consumption
    cli(kl) = clmat(ill(kl),ilh(kl));
    
    % optimal reform effort
    Upsi(kl) = Upsmat(ill(kl),ilh(kl));
    
    % optimal continuation value
    Evplli(kl) = Evpllmat(ill(kl),ilh(kl));
    Evplhi(kl) = Evplhmat(ill(kl),ilh(kl));
    
    end

    % update the profit function

    % get cubic spline
    Pl_spline = fnxtr(csaps(vl,Plnext),2);

    % set up function
    Plfun     = @(vvp) fnval(vvp,Pl_spline);

    %%% trick end %%%

    % Howard's improvement (policy function iterations)
    steps = iter*1;              % choose number of iterations

    % fix policy functions
    vphhi = vphh(ihh);
    vphli = vphl(ihl);
    vplhi = vplh(ilh);
    vplli = vpll(ill);

    for t=1:steps

        % compute continuation profits
        EPhhi = (1-Fh(vphhi)).*Phfun(vphhi)+Ih(vphhi)*(pi.*Phfun(vhath));
        EPhli = (1-Fl(vphli)).*Plfun(vphli)+Il(vphli)*(pi.*Plfun(vhatl));
        
        EPlhi = (1-Fh(vplhi)).*Phfun(vplhi)+Ih(vplhi)*(pi.*Phfun(vhath));
        EPlli = (1-Fl(vplli)).*Plfun(vplli)+Il(vplli)*(pi.*Plfun(vhatl));
        
        % iterate given the policy function
        Phnext = wh-chi+1/R*(ph*EPhhi+(1-ph)*EPhli);
        Plnext = wl-cli+1/R*(Upsi.*EPlhi+(1-Upsi).*EPlli);

        % update the profit function

        % get cubic spline
        Ph_spline = fnxtr(csaps(vh,Phnext),2);
        Pl_spline = fnxtr(csaps(vl,Plnext),2);

        % set up function
        Phfun = @(vvp) fnval(vvp,Ph_spline);
        Plfun = @(vvp) fnval(vvp,Pl_spline);
    end

    % compute distance
    dist = norm([Ph-Phnext,Plnext-Pl]);

    % update the profit function
    lax = .5;
    Ph  = lax*Phnext+(1-lax)*Ph;
    Pl  = lax*Plnext+(1-lax)*Pl;

    % display inner loop, progress
    fprintf('%4i \t %2.1e \n',iter,dist);

    % increase the iteration counter
    iter = iter+1;

end

fprintf('Done with planner allocation. \n');

%% 4. turn ex-post variables into ex-ante variables

% expected utility before the realization of phi
Evl = ((1-Fl(vl)).*vl+Il(vl)*(pi.*vhatl));
Evh = ((1-Fh(vh)).*vh+Ih(vh)*(pi.*vhath));

% expected profits before the realization of phi
EPl = ((1-Fl(vl)).*Plfun(vl)+Il(vl)*(pi.*Plfun(vhatl)));
EPh = ((1-Fh(vh)).*Phfun(vh)+Ih(vh)*(pi.*Phfun(vhath)));

% expected future promise
omegalli = Evplli;
omegalhi = Evplhi;

omegahli = Evphli;
omegahhi = Evphhi;

% save solution
xh = [vphhi,vphli,Ph,chi,ihh,ihl];
xl = [vplhi,vplli,Pl,cli,ilh,ill,Upsi];

% uncomment if you want to overwrite saved solution
%save(strcat('solution_gamma',int2str(gamma)),'vh','vl','xh','xl');

%% 5. switch to ex-ante variables of the planner problem

% expected promised utility
vvl = Evl;

% expected profits
PPl = EPl;

% future profits, recession
% derivative
P1l   = gradient(PPl,vvl);
P1pll = P1l(ill);
% level
Ppll  = PPl(ill);

% future profits, recovery
% derivative
P1h   = gradient(Ph,vh);
P1plh = P1h(ilh);
% level
Pplh  = Ph(ilh);

% derivative of the effort function
Ups1= beta./X11(Upsi);

%% 6. set up interpolation ready functions

% define spline interpolation as the standard
method = 'spline';

% threshold realization
Phifun = @(vv,h) h.*max(phih(1),min(phih(end),interp1(vh,Phih,vv,method)))...
            +(1-h).*max(phil(1),min(phil(end),interp1(vvl,Phil,vv,method)));
        
% consumption
Cfun = @(vv,h) h.*interp1(vh,chi,vv,method)+(1-h).*interp1(vvl,cli,vv,method);

% consumption as a function of phi
C    = @(pphi,h) h.*interp1(Phih,chi,pphi,method)+(1-h).*interp1(Phil,cli,pphi,method);

% reform effort
Upsfun = @(vv) max(0,min(1,interp1(vvl,Upsi,vv,method)));

% effort as a function phi
Ups    = @(pphi) interp1(Phil,Upsi,pphi,method);

% promised utility, recovery
Vhfun = @(vv,h) h.*interp1(vh,omegahhi,vv,method)+(1-h).*interp1(vvl,omegalhi,vv,method);

% promised utility, recovery as a function of phi
Vh    = @(pphi,h) h.*interp1(Phih,omegahhi,pphi,method)+(1-h).*interp1(Phil,omegalhi,pphi,method);

% promised utility, recession
Vlfun = @(vv,h) h.*interp1(vh,omegahli,vv,method)+(1-h).*interp1(vvl,omegalli,vv,method);

% promised utility, recession as a function of phi
Vl    = @(pphi,h) h.*interp1(Phih,omegahli,pphi,method)+(1-h).*interp1(Phil,omegalli,pphi,method);

%% 7. simulation with manual shocks

b0      = 1*wl;
ib0     = find(bgrid>=b0,1,'first');
ip0     = find(PPl>=Pil_SC(ib0),1,'first');
istar   = find(EVl_SC<=vvl(ip0),1,'first');
vstar   = vvl(ip0);
bstar   = bgrid(istar)/wl;

% pin down initial condition (zero profit contract, compared to SC)
v0 = vvl(ip0);

% figure out the zero profit promise

% note:
% - simulates only non-redundant economies and periods
nperiods = 15;                                      % number of periods to be simulated
npaths   = 4;                                       % number of paths to be simulated
st       = randi([1,S],npaths,nperiods);            % draws from uniform discrete distribution 

% initialize variables

% current promised-utility
vt = zeros(npaths,nperiods);

% promised-utility boom
vht = zeros(npaths,nperiods+1);

% promised-utility recession
vlt = zeros(npaths,nperiods+1);

% set initial conditions
vlt(:,1)   = v0;                 % zero profit value
vht(:,1)   = Vhfun(v0,0);        % irrelevant since we start in recession

% consumption
ct = zeros(npaths,nperiods);

% recovery probability
pt = zeros(npaths,nperiods);

% effort cost
xt = zeros(npaths,nperiods);

% normal times indicator
ht = zeros(npaths,nperiods+1);

% renegotiation indicator
rt = zeros(npaths,nperiods);

% renegotiation threshold
phit = zeros(npaths,nperiods);

%%% this is for the figures only %%%

% manual default shock series
Tf  = 15;    % final period
Tn  = 14;    % period where you get permanently out of the recession
Tr1 = 9;     % first renegotiation in recession
Tr2 = 11;     % second renegotiation in recession

% create 4 artificial series
% 1...no renegotiation and staying in recession forever
% 2...renegotiation1 and staying in recession forever
% 3...renegotiation1+2 and staying in recession forever
% 5...renegotiation1+2 and transition to normal times forever

st(:,1:Tf)    = S;         % maximum shock realization
st(2:end,Tr1) = 25;        % assign all included series the first shock realization 
st(3:end,Tr2) = 5;

% assign the normal times periods to the last series
ht(4,Tn:end)  = 1;

%%% end %%%

for t=1:nperiods
    vt(:,t)    = ht(:,t).*vht(:,t)+(1-ht(:,t)).*vlt(:,t);
    phit(:,t)  = Phifun(vt(:,t),ht(:,t));
    rt(:,t)    = ht(:,t).*(phih(st(:,t))<phit(:,t))+(1-ht(:,t)).*(phil(st(:,t))<phit(:,t));   % renegotiated proof contract
    ct(:,t)    = rt(:,t).*C(phil(st(:,t)),ht(:,t))+(1-rt(:,t)).*Cfun(vt(:,t),ht(:,t));
    pt(:,t)    = ht(:,t)*ph+(1-ht(:,t)).*(rt(:,t).*Ups(phil(st(:,t)))+(1-rt(:,t)).*Upsfun(vt(:,t)));
    xt(:,t)    = ht(:,t)*0+(1-ht(:,t)).*X(pt(:,t));
    vht(:,t+1) = rt(:,t).*Vh(phil(st(:,t)),ht(:,t))+(1-rt(:,t)).*Vhfun(vt(:,t),ht(:,t));
    vlt(:,t+1) = rt(:,t).*Vl(phil(st(:,t)),ht(:,t))+(1-rt(:,t)).*Vlfun(vt(:,t),ht(:,t));
end

%%% end: manipulated simulation

%%

%%% figure: 
%%% consumption, effort and expected utility over time
%%% 2 renegotiations, 1 permanent recovery

scrsz = get(0,'ScreenSize');
% the size of the figure (3/7) is important such that the strings appear in
% the slides with the appropriate size.
figure('Position',[scrsz(3)*1/9 scrsz(4)*1/20 scrsz(3)*4/9 ...
    scrsz(4)*2/3*1.1]);

% panel (c) 
% planner allocation: consumption and effort moral hazard
h = subplot(2,2,3);
pos    = get(h, 'position');
pos(1) = pos(2)-0.02;
pos(2) = pos(2)+0.06;
set(h,'position', pos);

% manual manipulation of the promised-utility series
ct(4,Tn-1) = ct(4,Tn);

% setup plot matrixes
%vtmat          = vt(:,1:Tf);
ctmat          = ct(:,1:Tf);
ptmat          = pt(:,1:Tf);
ptmat(ht(:,1:end-1)==1) = NaN;
vltmat         = vlt(:,2:Tf+1);
vhtmat         = vht(:,2:Tf+1);
Ttmat          = repmat(1:Tf,[npaths,1]);
Trmat          = [Tr1*ones(1,Tf);Tr2*ones(1,Tf);(Tn-1)*ones(1,Tf);Tf*ones(1,Tf)];
Trmatlag       = [ones(1,Tf);Trmat(1:npaths-1,1:Tf)];
mask           = find(Ttmat>Trmat);
masklag        = find(Ttmat<Trmatlag);
ctmat(mask)    = NaN;
ctmat(masklag) = NaN;
ptmat(mask)    = NaN;
ptmat(masklag) = NaN;
vltmat(mask)    = NaN;
vltmat(masklag) = NaN;
vhtmat(mask)    = NaN;
vhtmat(masklag) = NaN;

% mark recession with shaded areas
h = fill([1 1 Tn-1 Tn-1],[.35 1.05 1.05 .35],.9*[1,1,1],'edgecolor','none');
hold on;

% plot the time paths
[Ax,l1,l2] = plotyy((1:Tf),ctmat,(1:Tf),ptmat);
hold off;

%title('Moral hazard');
xlabel('Time');
ylabel(Ax(1),'Consumption')
ylabel(Ax(2),'Reform Effort')
legend([h l1(1) l2(1)],'Recession period','Consumption','Reform effort','Location','NorthWest');
set(l1,'color','b','marker','o','linestyle','-','linewidth',1.5);
set(l2,'color',[0,.5,0],'marker','x','linestyle','-','linewidth',1.5);

ylim(Ax(1),[.4 1.05]);
ylim(Ax(2),[.06 .12]);
xlim(Ax(1),[1,Tf]);
xlim(Ax(2),[1,Tf]);

set(Ax(1),'YTick',0.4:.1:1,'XTick',[1,5,10,13,15]);
set(Ax(1),'Box','Off');
set(Ax(2),'YTick',0.06:.01:.12);

% jumps consumption
line([Tr1,Tr1],[ct(1,Tr1) ct(2,Tr1)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');
line([Tr2,Tr2],[ct(2,Tr2) ct(3,Tr2)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');
line([Tn-1,Tn-1],[ct(3,Tn-1) ct(4,Tn-1)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');

% jumps reform effort
line([Tr1,Tr1],[pt(1,Tr1) pt(2,Tr1)],'parent',Ax(2),'Linewidth',1,'Linestyle',':','color',[0,.5,0]);
line([Tr2,Tr2],[pt(2,Tr2) pt(3,Tr2)],'parent',Ax(2),'Linewidth',1,'Linestyle',':','color',[0,.5,0]);

%%

% panel (d) 
% planner allocation: promised utility no moral hazard

% increase the space between subplots a bit
h = subplot(2,2,4);
pos    = get(h, 'position');
pos(1) = pos(1)+0.00;
pos(2) = pos(2)+0.06;
set(h,'position', pos);

% mark recession with shaded areas
fill([1 1 Tn-1 Tn-1],[-64.2 -61 -61 -64.2],.9*[1,1,1],'edgecolor','none');
hold on;

% plot the time paths
[Ax,l1,l2] = plotyy((1:Tn-1),vltmat(:,1:Tn-1),(1:Tf),vhtmat);
hold off;

%title('Moral hazard');
xlabel('Time');
ylabel(Ax(1),'Promised utility, Recession')
ylabel(Ax(2),'Promised utility, Recovery')
legend([l1(1) l2(1)],'Recession-cont.','Recovery-cont.','Location','NorthWest');
set(l1,'color','r','marker','o','linestyle','-','linewidth',1.5);
set(l2,'color','k','marker','d','linestyle','-','linewidth',1.5);

ylim(Ax(1),[-64.2 -61]);
ylim(Ax(2),[-54 -51]);
xlim(Ax(1),[1,Tf]);
xlim(Ax(2),[1,Tf]);

set(Ax(1),'YTick',-64:.5:-61,'XTick',[1,5,10,13,15]);
set(Ax(1),'Box','Off');
set(Ax(2),'YTick',-54:.5:-51);

% jumps promised utility, recession
line([Tr1,Tr1],[vlt(1,Tr1+1) vlt(2,Tr1+1)],'parent',Ax(1),'Linewidth',1.25,'Linestyle',':','color','r');
line([Tr2,Tr2],[vlt(2,Tr2+1) vlt(3,Tr2+1)],'parent',Ax(1),'Linewidth',1.25,'Linestyle',':','color','r');

% jumps promised utility recovery
line([Tr1,Tr1],[vht(1,Tr1+1) vht(2,Tr1+1)],'parent',Ax(2),'Linewidth',1.25,'Linestyle',':','color','k');
line([Tr2,Tr2],[vht(2,Tr2+1) vht(3,Tr2+1)],'parent',Ax(2),'Linewidth',1.25,'Linestyle',':','color','k');
line([Tn,Tn],[vht(3,Tn+1) vht(4,Tn+1)],'parent',Ax(2),'Linewidth',1.25,'Linestyle',':','color','k');

%%

% load no moral hazard solution
load('../../Two assets no moral hazard/Planner/simulation_figure')

% panel (a) 
% planner allocation: consumption and effort, no moral hazard
h = subplot(2,2,1);
pos    = get(h, 'position');
pos(1) = pos(1)-0.04;
set(h,'position', pos);

% mark recession with shaded areas
h = fill([1 1 Tn-1 Tn-1],[.9 .945 .945 .9],.9*[1,1,1],'edgecolor','none');
hold on;

% plot the time paths
[Ax,l1,l2] = plotyy((1:Tf),ctmat_SB,(1:Tf),ptmat_SB);
hold off;

%title('No moral hazard');
xlabel('Time');
ylabel(Ax(1),'Consumption')
ylabel(Ax(2),'Reform effort')
legend([h l1(1) l2(1)],'Recession period','Consumption','Reform effort','Location','NorthWest');
set(l1,'color','b','marker','o','linestyle','-','linewidth',1.5);
set(l2,'color',[0,.5,0],'marker','x','linestyle','-','linewidth',1.5);

ylim(Ax(1),[.9 .945]);
ylim(Ax(2),[.064 .085]);
xlim(Ax(1),[1,Tf]);
xlim(Ax(2),[1,Tf]);

set(Ax(1),'YTick',0.9:.01:0.945,'XTick',[1,5,10,13]);
set(Ax(1),'Box','Off');
set(Ax(2),'YTick',.065:.005:0.085);

% jumps consumption
line([Tr1,Tr1],[ct_SB(1,Tr1) ct_SB(2,Tr1)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');
line([Tr2,Tr2],[ct_SB(2,Tr2) ct_SB(3,Tr2)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');
line([Tn,Tn],[ct_SB(3,Tn) ct_SB(4,Tn)],'parent',Ax(1),'Linewidth',1,'Linestyle',':','color','b');

% jumps reform effort
line([Tr1,Tr1],[pt_SB(1,Tr1) pt_SB(2,Tr1)],'parent',Ax(2),'Linewidth',1,'Linestyle',':','color',[0,.5,0]);
line([Tr2,Tr2],[pt_SB(2,Tr2) pt_SB(3,Tr2)],'parent',Ax(2),'Linewidth',1,'Linestyle',':','color',[0,.5,0]);

%%

% panel (b) 
% planner allocation: promised utility, no moral hazard
% increase the space between subplots a bit
subplot(2,2,2);

% mark recession with shaded areas
fill([1 1 Tn-1 Tn-1],[-60 -63 -63 -60],.9*[1,1,1],'edgecolor','none');
hold on;

% plot the time paths
[Ax,l1,l2] = plotyy((1:Tn-1),vltmat_SB(:,1:Tn-1),(1:Tf),vhtmat_SB);
hold off;

%title('No moral hazard');
xlabel('Time');
ylabel(Ax(1),'Promised utility, Recession')
ylabel(Ax(2),'Promised utility, Recovery')
legend([l1(1) l2(1)],'Recession-cont.','Recovery-cont.','Location','NorthWest');
set(l1,'color','r','marker','o','linestyle','-','linewidth',1.5);
set(l2,'color','k','marker','d','linestyle','-','linewidth',1.5);

ylim(Ax(1),[-63 -60.5]);
ylim(Ax(2),[-56 -54]);
xlim(Ax(1),[1,Tf]);
xlim(Ax(2),[1,Tf]);

set(Ax(1),'YTick',-63:.5:-60.5,'XTick',[1,5,10,13]);
set(Ax(1),'Box','Off');
set(Ax(2),'YTick',-56:.5:-54);

% jumps promised utility, recession
line([Tr1,Tr1],[vlt_SB(1,Tr1+1) vlt_SB(2,Tr1+1)],'parent',Ax(1),'Linewidth',1.25,'Linestyle',':','color','r');
line([Tr2,Tr2],[vlt_SB(2,Tr2+1) vlt_SB(3,Tr2+1)],'parent',Ax(1),'Linewidth',1.25,'Linestyle',':','color','r');

% jumps promised utility recovery
line([Tr1,Tr1],[vht_SB(1,Tr1+1) vht_SB(2,Tr1+1)],'parent',Ax(2),'Linewidth',1.25,'Linestyle',':','color','k');
line([Tr2,Tr2],[vht_SB(2,Tr2+1) vht_SB(3,Tr2+1)],'parent',Ax(2),'Linewidth',1.25,'Linestyle',':','color','k');

%% 8. plot policy functions (planner allocation)

% figure out where the kink in omegalli is
kink   = find(omegalli<=omegalli(end),1,'first');
vvlkink = vvl(kink);

% figure 1: 
% - promise recession, promise recovery, 
% - consumption, effort
              
scrsz = get(0,'ScreenSize');
figure('Position',[scrsz(3)*1/8 scrsz(4)*1/8 scrsz(3)*1/2 ...
    scrsz(4)*3/5]);

% promised utility, recession
subplot(2,2,1)
hh = plot(vvl,[omegalli,vvl]);
set(hh(1),'Color','r','LineWidth',1.5);
set(hh(2),'Color','k','LineWidth',1','LineStyle','-.');
line([vvl(end),vvl(end)],[min(omegalli),max(omegalli)],...
         'LineWidth',1.25,'color','k','linestyle',':');
line([vvlkink+.025,vvlkink+.025],[min(omegalli),max(omegalli)],...
         'LineWidth',1.25,'color','k','linestyle',':');
title('Future promise, recession');
xlim([vvl(end)-.05,vvl(1)]);
ylim([omegalli(end)-.05,omegalli(1)]);
xlabel('Current promise, \nu'); ylabel('$\omega(\nu)$','interpreter','latex');
legend([hh(1) hh(2)],{'Recession-cont.','$45^{\circ}$-line'},...
    'interpreter','latex','Location','Best');

set(gca,'XTick',[vvl(end)-.01,vvlkink+.05,-63.5,-63,-62.5]);
xticks    = get(gca,'XTickLabel');
xticks{2} = '$\nu^{-}$';
xticks{1} = '$\underline{\nu}$';
set(gca,'XTickLabel',xticks,'ticklabelinterpreter','latex');
% %set(gca,'Box','Off');

% promised utility, recovery
h = subplot(2,2,2);
% manipulate position slightly
pos    = get(h, 'position');
pos(1) = pos(1)-0.035;
%pos(3) = pos(3)*2/3;
set(h,'position', pos);

hh = plot(vvl,[omegalhi,vvl]);
set(hh(1),'Color','b','LineWidth',1.5);
set(hh(2),'Color','k','LineWidth',1','LineStyle','-.');
hh(3) = line([vvl(end),vvl(end)],[min(vvl),max(omegalhi)],...
         'LineWidth',1.25,'color','k','linestyle',':');
hh(4) = line([vvlkink+.025,vvlkink+.025],[min(vvl),max(omegalhi)],...
         'LineWidth',1.25,'color','k','linestyle',':');
title('Future promise, recovery');
xlim([vvl(end)-.05,vvl(1)]);
ylim([vvl(end)-.05,omegalhi(1)]);
xlabel('Current promise, \nu'); ylabel('$\bar{\omega}(\nu)$','interpreter','latex');
legend([hh(1) hh(2)],{'Recovery-cont.','$45^{\circ}$-line'},...
    'interpreter','latex','Location','Best');

set(gca,'XTick',[vvl(end)-.01,vvlkink+.05,-63.5,-63,-62.5]);
xticks    = get(gca,'XTickLabel');
xticks{2} = '$\nu^{-}$';
xticks{1} = '$\underline{\nu}$';
set(gca,'XTickLabel',xticks,'ticklabelinterpreter','latex');
% %set(gca,'Box','Off');

% current consumption
subplot(2,2,3)

% smoothen plot
ccli = fnval(vvl,fnxtr(csaps(vvl,cli,.99999),2));

hh = plot(vvl,ccli,'Linewidth',1.5,'Color','b');
set(hh(1),'Color','r','LineWidth',1.5);
%set(hh(2),'Color',[0 .5 0],'LineWidth',1.5);
line([vvl(end),vvl(end)],[0,max(cli)+.05],...
        'LineWidth',1.25,'color','k','linestyle',':');
line([vvlkink+.025,vvlkink+.025],[0,max(cli)+.01],...
         'LineWidth',1.25,'color','k','linestyle',':');
title('Current consumption');
xlabel('Current promise, \nu'); ylabel('$c(\nu)$','interpreter','latex');
xlim([vvl(end)-.05,vvl(1)]);
ylim([min(cli)-.01,max(cli)+.01]);

set(gca,'XTick',[vvl(end)-.01,vvlkink+.05,-63.5,-63,-62.5]);
xticks    = get(gca,'XTickLabel');
xticks{2} = '$\nu^{-}$';
xticks{1} = '$\underline{\nu}$';
set(gca,'XTickLabel',xticks,'ticklabelinterpreter','latex');
% %set(gca,'Box','Off');

% reform effort
h = subplot(2,2,4);
% manipulate position slightly
pos    = get(h, 'position');
pos(1) = pos(1)-0.035;
%pos(3) = pos(3)*2/3;
set(h,'position', pos);

% smoothen plot
UUpsi = fnval(vvl,fnxtr(csaps(vvl,Upsi,.9998),2));

plot(vvl,UUpsi,'r','linewidth',1.5)
line([vvl(end) vvl(end)],[Upsi(end)-.002 max(Upsi)+.002],'Linewidth',1.25,'Linestyle',':','Color','k');
line([vvlkink+.025,vvlkink+.025],[Upsi(end)-.002 max(Upsi)+.002],...
         'LineWidth',1.25,'color','k','linestyle',':');
title('Reform effort')
ylabel('$\Upsilon(\bar{\omega}(\nu)-\omega(\nu))$','interpreter','latex');
xlabel('Current promise, \nu');
xlim([vvl(end)-.05,vvl(1)]);
ylim([Upsi(end)-.002,max(Upsi)+.002]);

set(gca,'XTick',[vvl(end)-.01,vvlkink+.05,-63.5,-63,-62.5]);
xticks    = get(gca,'XTickLabel');
xticks{2} = '$\nu^{-}$';
xticks{1} = '$\underline{\nu}$';
set(gca,'XTickLabel',xticks,'ticklabelinterpreter','latex');
